#!/usr/bin/env /usr/bin/python3
# PYTHON_ARGCOMPLETE_OK

"""
DataFrame:
   "StationId": station_id,
   "mm": res,
   "Date": [datetime(year, month, 1) + timedelta(hours=h)
"""

import time
start_time = time.perf_counter()
import argparse
# eval "$(register-python-argcomplete ./hydro.py)"
# https://github.com/kislyuk/argcomplete/issues/369
import argcomplete
parser = argparse.ArgumentParser()
parser.add_argument('--sid', nargs=1, default=None)
parser.add_argument('--map', action='store_true')
parser.add_argument('--full-series', action='store_true', default=False) # bop = beginning of project; extended data set or not
parser.add_argument('--noise', default=0.2, type=float)
parser.add_argument('--plot-all-series', action='store_true')
parser.add_argument('--plot-some-series',  nargs="*")
parser.add_argument('--cumulative-series', action='store_true')
parser.add_argument('--animate-cities', action='store_true')
parser.add_argument('--pearson', action='store_true')
parser.add_argument('--rain-distribution', action='store_true')
parser.add_argument('--analyze-quality', action='store_true')
parser.add_argument('--import-data', action='store_true')
parser.add_argument('--find-storms', action='store_true')
parser.add_argument('--export-storms', action='store_true')
parser.add_argument('--precipitation-per-month', action='store_true')
parser.add_argument('--precipitation-cumulative', action='store_true')
parser.add_argument('--test-emd', action='store_true')
parser.add_argument('--fit-storm-iat', action='store_true')
parser.add_argument('--fit-proba', action='store_true')
parser.add_argument('--plot-correlation', action='store_true')
parser.add_argument('--make-idf-curves', const=True, nargs="?")
parser.add_argument('--max-avg-storms', const=True, nargs="?")
parser.add_argument('--compute-storms-separation-chart', action='store_true')
parser.add_argument('--plot-irm-idf', default=None, help="Give the station name used by IRM. Example: Bastogne")
parser.add_argument('--plot-irm-idf-simple', default=None, help="Give the station name used by IRM. Example: Bastogne")
parser.add_argument('--bare-ts', action='store_true')
parser.add_argument('--noise-analysis', action='store_true', help="Noise analysis")
argcomplete.autocomplete(parser)

import re
import io
import glob
import locale
import random
import os.path
from calendar import monthrange
from math import sin, cos, sqrt, floor, ceil
from datetime import datetime, timedelta, date

import dateparser
from dateutil.relativedelta import relativedelta

import numpy as np
import numpy_groupies as npg
import pandas as pd
from scipy.stats.stats import pearsonr
from scipy import stats

import matplotlib.pyplot as plt
from matplotlib.dates import DateFormatter
from matplotlib.animation import FuncAnimation
from matplotlib.colors import LogNorm, Normalize
from matplotlib import cm
import seaborn as sns

from tqdm import tqdm
from tools import compute_storms, load_data, compute_idf_curves, plot_idf, compute_simple_IDF, compute_irm_idf, IRM_DURATIONS, Station, load_station_data, NAN_THRESHOLD, REGULAR_DATA_SET
from tools import BALMORAL,BASTOGNE,BATTICE,BIERSET,BUTGENBACH,CINEY,EREZEE,FLAMIERGES,GEMMENICH,JALHAY,LANAYE,LANDENNE,LOUVEIGNE,MARCHE,MODAVE,NASSOGNE,TERNELL,ORTHO,OUFFET,RACHAMPS,REULAND,ROBERTVILLE,ROCHEFORT,SART_TILMAN,SAINT_HUBERT,SANKT_VITH,TAILLES,VIELSALM,WAREMME
from dl_idf import ins_read_excel

print(f"Loaded imports in {time.perf_counter() - start_time:.1f} seconds")

locale.setlocale(locale.LC_ALL,'')

import platform
if platform.system() == "Windows":
    DATAPATH=r"c:\Users\Administrator\Dropbox\uliege\tfe"
else:
    DATAPATH="/mnt/data2/uliege/tfe"


def zero_runs(a):
    # From https://stackoverflow.com/questions/24885092/finding-the-consecutive-zeros-in-a-numpy-array

    # Create an array that is 1 where a is 0, and pad each end with an extra 0.
    iszero = np.concatenate(([0], np.equal(a, 0).view(np.int8), [0]))
    absdiff = np.abs(np.diff(iszero))
    # Runs start and end where absdiff is 1.
    ranges = np.where(absdiff == 1)[0].reshape(-1, 2)
    return ranges

def non_zero_runs(a):
    # From https://stackoverflow.com/questions/24885092/finding-the-consecutive-zeros-in-a-numpy-array
    # Create an array that is 1 where a is 0, and pad each end with an extra 0.
    iszero = np.concatenate(([0], np.not_equal(a, 0).view(np.int8), [0]))
    absdiff = np.abs(np.diff(iszero))
    # Zero-runs start and end where absdiff is 1.
    ranges = np.where(absdiff == 1)[0].reshape(-1, 2)
    return ranges


def dmy(dt):
    if type(dt) != datetime:
        return "?"
    else:
        return dt.strftime("%d/%m/%Y")

def human_date(s):
    # Specify the timezone to avoid PytzUsageWarning: The
    # localize method is no longer necessary, as this time
    # zone supports the fold attribute (PEP 495).
    return dateparser.parse(
        s,
        settings={'PREFER_DAY_OF_MONTH': 'first',
                  'TIMEZONE': 'Europe/Brussels'})

# class Station:
#     def __init__(self, row):
#         self.sid = row.StationId
#         self.x = row.X / 10000.0
#         self.y = row.Y / 10000.0
#         self.name = row.Name
#         self.full_name = None

#     def distance(self, s):
#         return sqrt((self.x-s.x)**2 + (self.y-s.y)**2)

#     def __str__(self):
#         return self.name


# def load_station_data(fpath):
#     # station meta info are collected during the download.
#     # don't collect them here.

#     # Read station data
#     STATION_DATA = pd.read_csv(fpath, header=0)
#     # print(STATION_DATA)
#     # print(STATION_DATA.dtypes)
#     stations = dict()
#     for ndx, row in STATION_DATA.iterrows():
#         stations[int(row.StationId)] = Station(row)

#     # Make python code
#     # a = [(row.Name, row.StationId) for ndx, row in STATION_DATA.iterrows()]
#     # for n, sid in sorted(a):
#     #     print(f"{n} = {sid}")
#     return stations


def load_series(extended=False):
    if extended:
        STATIONS_SET=[80630015,35340015,10430015,95960015,97810015,97940015,55960015,65370015,99480015,56490015,84680015,96320015,72070015,36490015,94690015,91370015,10810015,69670015,73690015,29930015,81570015,38850015,67120015,67180015,86870015,81380015,92880015,83480015,24590015,28920015,70480015,61470015,81890015,95740015,86770015,52840015,76780015,15840015,65500015,32770015,57570015,71680015,83880015,76290015,23480015,10570015,66570015,70870015,70180015,95880015,72280015,75770015,73950015,73350015,34760015,96980015,85380015,61280015,98160015,70160015,37170015,15790015,92150015,78650015,60480015,69580015,85180015,36280015,36470015,81280015,61680015,99220015,70030015,97970015,23880015,78880015,79670015,19970015,74850015,70680015,65380015,99150015,96520015,96170015,68580015,64970015,97430015,35720015,19540015,18980015,72960015,68480015,94360015,55780015]
        print("Loading extended dataset")
    else:
        STATIONS_SET=REGULAR_DATA_SET
        print("Loading regular dataset")

    TITLE_RE = re.compile(r"title: \{[^t]+text: '(.+)\((.+)\) - Précipitations \(mm\)'")
    DATE_RE = re.compile(r"pointStart: Date.UTC\(([0-9]+), ([0-9]+), ([0-9]+)\),")
    DATA_RE = re.compile(r"var data = \[([ .,0123456789null]+)\];")
    BDATA_RE = re.compile(r"var bdata = \[([ .,0123456789null]+)\];")
    all_data = pd.DataFrame()

    for fname in tqdm(glob.glob(f"{DATAPATH}/hydro_tab*.html")):
        _,_,station_id, year, month = fname.split("_")

        year = int(year)
        month = int(month.replace(".html",""))
        station_id = int(station_id)

        if station_id not in STATIONS_SET:
            continue

        with open(fname, "r") as fin:
            html = fin.read()

            # # We get that station information from each file
            # # of course, we'll get the same station a lot of time.
            # # FIXME not clean, should do some file apart.
            # if station_id not in stations:
            #     s = TITLE_RE.search(html)
            #     if s:
            #         stations[station_id].full_name = f"{s.group(1).strip()}/{s.group(2).strip()}"

            # continue

            html_tables = pd.read_html(html, match='.+')

            nbdays = monthrange(year, month)[1]

            # F : column first

            if len(html_tables) >= 4:
                res = html_tables[4].to_numpy()[0:24,1:nbdays+1].astype('float').reshape(-1,order='F')

                df = pd.DataFrame({
                    "StationId": station_id,
                    "mm": res,
                    "Date": [datetime(year, month, 1) + timedelta(hours=h)
                             for h in range(24*nbdays)]
                })

                all_data = all_data.append(df)
            else:
                print(f"Can't parse {fname}")

    all_data = all_data.sort_values(by = ['StationId', 'Date'])
    return STATIONS_SET, all_data


def series_to_numpy(dataframe, prefix=None):
    n = data.to_numpy()
    c_stationid = n[:,0].astype(int)
    c_mm = n[:,1].astype(float)
    c_date = n[:,2].astype('datetime64')

    if prefix is None:
        pfx = f"{DATAPATH}/np_"
    else:
        pfx = f"{DATAPATH}/np_{prefix}_"

    np.save(f"{pfx}station_id.npy", c_stationid)
    np.save(f"{pfx}mm.npy", c_mm)
    np.save(f"{pfx}date.npy", c_date)



def filter_by(data, sid, begin="1/1/2000", end="31/12/2022", col="mm"):
    #print(f"Filter by sid={sid}, begin={begin} end={end}")

    if type(begin) == str:
        begin = human_date(begin)
    if type(end) == str:
        end = human_date(end)

    f = True
    if sid is not None:
        f = data["StationId"] == sid
    if begin is not None:
        f = f & (data['Date'] >= begin)
    if end is not None:
        f = f & (data['Date'] < end)

    filtered_data = data[f]
    if col in  ("mm", "Date", "StationId"):
        r = np.array(filtered_data[col])
        # Make it a column vector
        r = r[...,np.newaxis]
        ret = r

    elif col == "all":
        ret = filtered_data

    print(f"Filter by sid={sid}, begin={dmy(begin)} end={dmy(end)}. Result: from {dmy(np.min(filtered_data['Date']))} to {dmy(np.max(filtered_data['Date']))}")
    return ret



def plot_cities(data, stations):

    fig, ax = plt.subplots(1, 1, figsize=(16, 8), dpi=100)

    img = plt.imread("carte_belgique.png")
    ax.imshow(img) #, extent=[-5, 80, -5, 30])
    # Sart tilman pos
    XOFFSET = 861
    YOFFSET = 143
    XSCALE = (740 - XOFFSET) / (stations[ROCHEFORT].x - stations[SART_TILMAN].x)
    YSCALE = (386 - YOFFSET) / (stations[ROCHEFORT].y - stations[SART_TILMAN].y)
    # plt.scatter([XOFFSET + (s.x - stations[SART_TILMAN].x)*XSCALE for s in stations.values()],
    #             [YOFFSET + (s.y - stations[SART_TILMAN].y)*YSCALE for s in stations.values()],
    #             c="red")
    for sid, s in stations.items():
        if sid in REGULAR_DATA_SET:
            # plt.annotate(f"{s.name}", (XOFFSET + (s.x - stations[SART_TILMAN].x)*XSCALE,
            #                            YOFFSET + (s.y - stations[SART_TILMAN].y)*YSCALE))
            ax.scatter([XOFFSET + (s.x - stations[SART_TILMAN].x)*XSCALE],
                     [YOFFSET + (s.y - stations[SART_TILMAN].y)*YSCALE], c="black")
            ax.text(XOFFSET + (s.x - stations[SART_TILMAN].x)*XSCALE,
                     YOFFSET + (s.y - stations[SART_TILMAN].y)*YSCALE,
                     f"{s.name}",
                     ha="right", va="bottom",
                     bbox=dict(boxstyle="square,pad=0.1", fc="white"))
    #hide x-axis
    ax.get_xaxis().set_visible(False)

    #hide y-axis
    ax.get_yaxis().set_visible(False)

    plt.savefig(f"map_cities.png", format="png", dpi=300) # bbox_inches="tight",
    plt.show()


def make_anim(data, stations):
    # https://stackoverflow.com/questions/9401658/how-to-animate-a-scatter-plot

    #fig = plt.figure(figsize=(7,7))
    fig, axs = plt.subplots(1,1,figsize=(6,6))
    ax = axs
    ln = ax.scatter([s.x for s in stations.values()],
                    [s.y for s in stations.values()])
    title = ax.text(0.3,0.95, f"{datetime(2021,1,1)+ timedelta(hours=13)}", bbox={'facecolor':'w', 'alpha':0.5, 'pad':5}, transform=ax.transAxes, ha="right", fontsize='x-small')
    for s in stations.values():
        ax.annotate(f"{s.name}", (s.x, s.y), fontsize='x-small', ha="center")

    plt.show()

    nb_frames = 24*365 # 24 hours of 365 days

    rainy_hours = []
    for h in tqdm(range(365*24)):
        dh = datetime(2021,1,1)+ timedelta(hours=h)
        sx, sy, ss = rain_on_hour(data, stations, dh)
        if sum(ss) > 1:
            rainy_hours.append((dh, sx, sy, ss))


    nb_frames = len(rainy_hours)
    def update(h):
        print(f"{h}/{nb_frames}")
        # d = datetime(2021,1,1)+ timedelta(hours=h)
        # sx, sy, ss = rain_on_hour(data, stations, d)
        dh, sx, sy, ss = rainy_hours[h]
        ln.set_offsets(np.transpose(np.array([sx, sy])))
        ln.set_sizes((np.array(ss)*4)**2)
        title.set_text(f"{dh}")
        return ln, title

    start_time = time.perf_counter()
    anim = FuncAnimation(fig, update, frames=range(nb_frames),
                         blit=True, interval=0)
    #plt.show()
    anim.save('movie.mp4', writer = 'ffmpeg', fps = 10)
    print(f"Rendered {nb_frames} frames in {time.perf_counter() - start_time:.1f} seconds")


def rain_on_hour(data, stations, d):
    rainy_hour = data[ data["Date"] == d]

    sx = []
    sy = []
    ss = []
    for ndx, row in rainy_hour.iterrows():
        sid = row.StationId
        sx.append(stations[sid].x)
        sy.append(stations[sid].y)
        ss.append(row.mm)
    return sx, sy, ss

def animate_cities(data, stations, h):
    d = datetime(2021,7,14)+ timedelta(hours=h)
    rainy_hour = data[ data["Date"] == d]

    sx = []
    sy = []
    ss = []
    for ndx, row in rainy_hour.iterrows():
        sid = row.StationId
        sx.append(stations[sid].x)
        sy.append(stations[sid].y)
        ss.append(row.mm)

    # pp = rainy_hour.loc[rainy_hour['StationId'] == RACHAMPS]
    # if not pp.empty:
    #     pp = pp.head(1).mm

    plt.scatter(sx,sy,s=np.array(ss)*10)

    for ndx, row in rainy_hour.iterrows():
        s = stations[row.StationId]
        plt.annotate(f"{s.name} {s.sid}", (s.x, s.y))
    plt.show()



def plot_all_series(all_data, stations):
    for sid in pd.unique(all_data['StationId']):
        s = all_data[all_data['StationId'] == sid]
        t = s[s["mm"] > 0]
        #plt.bar([x.to_pydatetime() for x in t["Date"]], t["mm"], label=sid, width=1/(24*60))
        plt.scatter(t["Date"], t["mm"], label=stations[sid].name)

    plt.xlabel("Time")
    plt.ylabel("Precipitations (mm)")
    plt.show()



def plot_some_series(all_data, stations, sids, start_date, end_date):

    fig, axs = plt.subplots(len(sids))

    for ndx, sid in enumerate(sids):
        s = all_data[(all_data['StationId'] == sid)
                     & (data['Date'] >= start_date)
                     & (data['Date'] < end_date)]

        # Stem plot
        #t = s[s["mm"] > 0]
        t = s

        axs[ndx].stem(t["Date"], t["mm"], label=stations[sid].name,  markerfmt=' ')
        axs[ndx].legend()

    plt.xlabel("Time")
    plt.ylabel("Precipitations (mm)")
    plt.legend()
    plt.show()


def plot_cumulative(all_data, stations, sids):

    for ndx, sid in enumerate(sids):
        s = all_data.loc[ (all_data['StationId'] == sid)
                          & (all_data['Date'] >= datetime(2021, 7, 23))
                          & (all_data['Date'] < datetime(2021, 8, 1)) ]
        t = np.cumsum(s["mm"])
        plt.plot(s["Date"], t, label=stations[sid].name)

    plt.xlabel("Time")
    plt.ylabel("Precipitations (mm)")
    plt.legend()
    plt.show()

def rain_distribution(data, stations):
    sa = stations.values()
    nbs = len(sa)

    print(f"{nbs} stations as a {ceil(sqrt(nbs))}*{ceil(sqrt(nbs))} grid")
    fig, axs = plt.subplots(ceil(sqrt(nbs)), ceil(sqrt(nbs)))
    axs = axs.ravel()
    max_bin = 0
    max_nb_bins = 0

    for axe, s in enumerate(sa):
        v = np.array(data[data["StationId"] == s.sid]["mm"])
        v = np.log(v[v > 0] + 1)

        if len(v) >= 1:
            #np.histogram(v, bins=20)
            bins, bin_edges, _ = axs[axe].hist(v, label=s.name)
            max_bin = max(max_bin, np.max(bins))
            max_nb_bins = max(max_nb_bins, np.max(bin_edges))
            axs[axe].legend()

    for a in axs:
        a.set_xlim([0, max_nb_bins])
        a.set_ylim([0, max_bin])

    fig.suptitle("Distribution of log( (precipitations > 0) + 1)\nAll data")
    plt.show()





def compute_pearson_correlation(data, stations):

    m = None
    s_names = []
    kept_stations = []
    ref_shape = None
    START, END = 2010, 2022
    for s in sorted(stations.values(),key=lambda s:s.x):
        v = np.array(data[(data["StationId"] == s.sid)
                          & (data['Date'] >= datetime(START, 1, 1))
                          & (data['Date'] < datetime(END, 1, 1))]["mm"])
        if len(v) == 0:
            continue

        #np.nan_to_num(v, copy=False, nan=0.0, posinf=None, neginf=None)

        if ref_shape is None:
            ref_shape = v.shape[0]

        if v.shape[0] == ref_shape: # (365+366)*24 For 1 year: 8772

            sd = np.std(v)
            if abs(sd) > 0:
                s_names.append(s.name)
                kept_stations.append(s)
                if m is None:
                    m = v
                else:
                    m = np.vstack([m,v])
            else:
                print(f"StdDev = {sd} for {s.name}")

        else:
            print(f"Strange {s.name}, {v.shape[0]} valuesnot equal to {ref_shape}")

    # FIXME Always > 0 ?
    # All data of one station correleated to all data of another station
    # Since all data are positive
    print(m.shape)
    # 21 rows = 21 variables = 21 stations
    # 17544 observations per row.
    # Persaon compute correlation between variabes (ie stations)
    # If two varaibles are equal, then corr == 1
    # If we change an observation then corr decreases
    # It's positive because when it rains a lot somwhere, it rains
    # a lot somewhere else. This in itself is a strong indication
    # of correlation at the global scale.
    mcorr = np.corrcoef(m)
    print(mcorr)

    fig, (ax1, ax2) = plt.subplots(1, 2, sharey=True, figsize=(16, 8), dpi=100)
    fig.suptitle(f"Pearson correlation of rain at $t$ h.\nAll stations, data from {START} to {END}")

    ax1.set_title('Correlation (Pearson)')
    mask = np.zeros_like(mcorr)
    mask[np.triu_indices_from(mask)] = True
    sns.heatmap(mcorr, xticklabels=s_names, mask=mask, yticklabels=s_names, ax=ax1, square=True)

    # Compute distance matrix

    sa=kept_stations
    m = np.zeros((len(sa),len(sa)))
    s_names = [s.name for s in sa]
    for i in range(len(sa)):
        a = sa[i]
        for j in range(i):
            b = sa[j]
            m[i,j] = a.distance(b)

    #m = m - np.max(m)
    ax2.set_title('Distance')
    ax = sns.heatmap(m, xticklabels=s_names, mask=mask, yticklabels=s_names, ax=ax2, cmap='magma_r', square=True)
    plt.savefig(f"distance_rain_correlation.png", format="png") #, dpi=300) # bbox_inches="tight",
    plt.show()


    plt.figure()
    fig.suptitle("Pearson correlation * distance")
    #likeness = np.abs(np.abs(mcorr) / np.max(np.abs(mcorr)) - np.abs(m / np.max(np.abs(m))))
    likeness = np.abs(mcorr * (np.max(m) - m + 1))
    sns.heatmap(likeness, xticklabels=s_names, mask=mask, yticklabels=s_names, cmap='magma_r', square=True)
    plt.show()


    # sids = [s.sid for s in stations]
    # for i in range(len(sids)):
    #     a = data[data["StationId"] == sids[i]]
    #     for j in range(i):
    #         b = data[data["StationId"] == sids[j]]
    #         coeff, p_value = pearsonr(a,b)


def broad_select_bang(data, stations):
    stations_to_drop = set()
    for sid in stations:
        ds = data[data["StationId"] == sid]
        nb_nans = np.count_nonzero(np.isnan(ds['mm']))
        if nb_nans > 100:
            stations_to_drop.add(sid)

    print(f"Dropping {', '.join([stations[sid].name for sid in stations_to_drop])}")
    for sid in stations_to_drop:
        stations.pop(sid)
    data.drop( data[data["StationId"].isin(stations_to_drop)].index, inplace=True)

    data.fillna({"mm":0}, inplace=True)

    print(f"{len(data):n} records left after drops and NaN's clear. Max mm of precipitation per hour is {np.max(data['mm'])}.")
    return data



def analyze_quality(data, stations):
    stations = list(stations.values())
    start_all = data["Date"].min()
    end_all = datetime.today()
    r = relativedelta(end_all, start_all)
    nb_months = r.months + (12*r.years)
    print(f"Analysing {nb_months} months, {len(stations)} stations")

    with open("data_quality.csv","w") as fout:
        fout.write(f"Name,First-Last Year,NaN's\n")
        for s in sorted(stations):
            ds = data[(data["StationId"] == s.sid)]
            if len(ds) == 0:
                continue
            nb_nans = np.count_nonzero(np.isnan(ds['mm']))
            fout.write(f"{s.name.title()},{ds['Date'].min().year}-{ds['Date'].max().year},{nb_nans or '-'}\n")

    t = pd.read_csv("data_quality.csv")
    mask = (t["First-Last Year"] == "2002-2021") & (t["NaN's"].replace("-","0").astype(int) < NAN_THRESHOLD)
    t["First-Last Year"] = t["First-Last Year"].replace("2002-2021","")
    good = t[mask]
    bad = t[~mask]
    t = pd.concat([good, bad], axis=0)
    t.to_csv("t2.csv", index=False)

    print("Wrote data_quality.csv and t2.csv")

    for s in sorted(stations):
        ds = data[(data["StationId"] == s.sid)]
        if len(ds) == 0:
            continue
        nb_nans = np.count_nonzero(np.isnan(ds['mm']))
        print(f"{s.name:20s}: {ds['Date'].min()} {ds['Date'].max()} NAN:{nb_nans}")

        if nb_nans < NAN_THRESHOLD:
            continue

        v = np.array(data[(data["StationId"] == s.sid)]["mm"])

        stat = ""

        for i in range(nb_months):
            start = start_all + relativedelta(months=i)
            end = start_all + relativedelta(months=i+1)

            v = np.array(data[(data["StationId"] == s.sid)
                              & (data['Date'] >= start)
                              & (data['Date'] < end)]["mm"])

            if v.shape[0] == 0:
                stat += "/"
            else:
                nans = np.count_nonzero(np.isnan(v))

                if nans > 24*28:
                    stat += "N"
                elif nans > 1:
                    stat += "n"
                else:
                    stat += "."
        print(stat)

def array_to_file(a,path):
    print(f"Writing {len(a)} (min={min(a)}, max={max(a)}) elements to {path}")
    print(str(list(a))[1:-1],
          file=open(path,"w"))


def fit_storm_iat2(data, stations, mm_start, mm_end, step_size, ax): # iat = interarrival time
    #v = np.array(data[(data["StationId"] == ROBERTVILLE)]["mm"])
    v = np.array(data["mm"])

    ex_zone = 3
    shift = 1
    ZOOM=10 # used to make fractional rain measures integer (+ around operation).

    XEX_MIN = 0
    XEX_MAX = ex_zone*ZOOM
    YEX_MIN = 0
    YEX_MAX = ex_zone*ZOOM
    TICKS=10

    r = np.round(v * ZOOM).astype(int)

    # Build the grid of probabilities

    g = np.zeros( (np.max(r)+1, np.max(r)+1) )
    before, after = r[0:-shift], r[shift:]
    for i in range(len(r)-shift):
        # before = X axis; y axis = after.
        g[after[i],before[i]] += 1

    # Show actual probabilities distributions (mass, not density)


    # 0.1 because we look at what happens when it *rains* at t.
    # The non-rain event is trickier to handle so I feel it's
    # safer to put it aside.
    for i in np.arange(mm_start,mm_end,step_size): # 0mm to 5mm at t.
        s,e = round(i*ZOOM), round((i+step_size)*ZOOM)
        print(f"i:{i} s:{s} e:{e}")
        vv = np.sum(g[:,s:e],axis=1)
        vv = vv / (np.sum(vv) or 1)
        ax.plot(vv[0:XEX_MAX],
                #ax.plot(np.convolve(vv[0:XEX_MAX], np.ones(3)/3, 'valid'),
                label=f"Average of [{i:.1f}, {i+step_size:.1f}[ mm at $t$")

    t = np.arange(XEX_MIN,XEX_MAX,((XEX_MAX-XEX_MIN)//TICKS or 1))
    ax.set_xticks(t, labels=t/ZOOM)
    ax.set_xlabel(f"Rain at $t+{shift}h$ (mm)")
    ax.set_ylabel(f"P(Rain at $t+{shift}h$ | rain at $t$)")
    ax.set_title(f"If it rains at $t$ what happens at $t+{shift}h$ ?")
    ax.legend()

def cart_prod(x,y):
    return np.transpose([np.tile(x, len(y)), np.repeat(y, len(x))])


def prob_array(data):
    # Return the probability associated to each row
    # of the array. It is the probability of seeing that
    # row in the array. So the sum of returned probabilities is
    # not one. For ex: 1,1,2 will have p=0.66,0.66,0.33.

    # Count each distinct row
    _, inv_ndx, cnt = np.unique(
        data,
        axis=0,
        return_inverse=True,
        return_counts=True) # count (S1=s1, S2=s2, S3=s3) pairs

    # The count is now the probability
    p = cnt/np.sum(cnt)

    # To each data row will correspond one probability
    # (the probability of that row appearing relative
    # to the rest of the dataset).
    r = np.zeros((len(data), 1))
    for i in tqdm(range(len(p))):
        r[inv_ndx == i,0] = p[i]

    return r

def fit_proba(data, stations):
    # Reshape to make column vectors
    subs, LOW, LIM, BEGIN_DATE = [MARCHE, NASSOGNE, ROCHEFORT], 0, 5, '1/1/2007'
    #subs, LOW, LIM, BEGIN_DATE = [LANAYE, CINEY, BASTOGNE], 0, 5, '1/1/2007'

    s1 = filter_by(data, subs[0], begin=BEGIN_DATE)
    s2 = filter_by(data, subs[1], begin=BEGIN_DATE)
    s3 = filter_by(data, subs[2], begin=BEGIN_DATE)

    assert LIM > LOW
    assert s1.shape == s2.shape == s3.shape

    s1s2s3 = np.hstack((s1,s2,s3))

    # Compute probabilities --------------------------

    prob_s1s2s3 = prob_array(s1s2s3)

    s2s3 = s1s2s3[:,1:3]
    prob_s2s3 = prob_array(s2s3)

    # P(s1 at t | s2,s3 at t)
    cond_prob = prob_s1s2s3 / prob_s2s3
    cond_prob /= np.sum(cond_prob)

    # Now we marginalize t
    # This means, we'll sum all the s1 corresponding to s2/s3 pair.
    # It's not marginalizing s1 because s2/s3 probabilities don't change.

    msk = ((s2 <= LIM) & (s3 <= LIM) & (s2 + s3 >= LOW)).flatten()
    print(msk.shape)
    s2s3_t_out, inv_ndx = np.unique(s2s3[msk,:], axis=0, return_inverse=True)
    print(inv_ndx.shape)
    print(cond_prob.shape)
    cp1 = (s1[msk,:] > 0).astype(int) # Some rain (1 = rain, 0 = no rain)
    cond_prob_t_out = npg.aggregate(
        inv_ndx,
        cp1.flatten(), func='mean', fill_value=0)
    cond_prob_t_out /= np.sum(cond_prob_t_out)
    print(f"{np.sum(cond_prob)},  {np.sum(cond_prob_t_out)}")

    # Nb observations s2/s3
    z2, cnt2 = np.unique(s2s3[msk,:], axis=0, return_counts=True) # make charting faster
    df = pd.DataFrame({'s2':z2[:,0],
                       's3':z2[:,1],
                       'nb_obs':cnt2})
    grid_nb_obs = pd.pivot_table(df, values='nb_obs',index=["s2"], columns=["s3"], fill_value=0, aggfunc=sum)


    # ------------------------------------------------

    # all_data = S1,S2,S3 and P(S1|S2,S3)
    all_data = np.hstack((s1s2s3, cond_prob))
    z, cnt = np.unique(all_data, axis=0, return_counts=True) # make charting faster
    z[:,3] *= cnt # But still count the probabilities correctly
    z[:,3] /= np.sum(z[:,3]) # Back to probability distribution

    m = (z[:,1] <= LIM) & (z[:,2] <= LIM) & (z[:,0] < LIM) & (z[:,1] + z[:,2] >= LOW)

    if True:
        # When it rains in s2 and s3, we count all rain in s1 for all t's.
        # Note that if the s1 collected rains are : [0,0,0,0,0,1,0,0,2]
        # the total is 3. It may be comparable to other measures of s1 (at other s2/s3's)
        # but it doesn't give any information on the probability that it will
        # rain at s1 if it rains at s2/s3.
        # To get a better idea, we take the mean of the observations at s2/s3
        # Of course, that mean is very skewed. For some s2/s3 we will
        # have hundreds of observations, whereas for other we'll have only one

        m2 = (all_data[:,1] <= LIM) & (all_data[:,2] <= LIM) & (all_data[:,0] < LIM) & (all_data[:,1] + all_data[:,2] >= LOW)
        z2, cnt2 = np.unique(all_data[m2,1:3], axis=0, return_counts=True) # make charting faster
        df = pd.DataFrame({'s2':z2[:,0],
                           's3':z2[:,1],
                           'nb_obs':cnt2})
        pt = pd.pivot_table(df, values='nb_obs',index=["s2"], columns=["s3"], fill_value=0, aggfunc=sum)

        df = pd.DataFrame({'s2':all_data[m2,1],
                           's3':all_data[m2,2],
                           's1':all_data[m2,0]})
        pt2 = pd.pivot_table(df, values="s1",index=["s2"], columns=["s3"], fill_value=np.nan, aggfunc='mean')

        fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(16, 8))
        fig.suptitle(f"{stations[subs[0]].name} given {stations[subs[1]].name} & {stations[subs[2]].name}")
        ax = axes[0]
        sns.heatmap(pt, cmap="viridis", square=True, norm=LogNorm(), ax=ax)
        ax.invert_yaxis()
        ax.set_title(f"# observations (log-scale)")
        ax.set_xlabel(f"Precipitations in {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations in {stations[subs[2]].name} (mm/h)")

        ax = axes[1]
        sns.heatmap(pt2, square=True, cmap="viridis", ax=ax)
        ax.invert_yaxis()
        ax.set_title(f"Mean of observed precipitations (mm)")
        ax.set_xlabel(f"Precipitations in {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations in {stations[subs[2]].name} (mm/h)")
        plt.savefig(f"two_correlation_data.pdf", format="pdf", bbox_inches="tight", dpi=600)
        plt.show()


    if True:

        fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(16, 8))
        fig.suptitle(f"{stations[subs[0]].name} given {stations[subs[1]].name} & {stations[subs[2]].name}")
        ax = axes[0]
        sns.heatmap(grid_nb_obs, cmap="viridis", square=True, norm=LogNorm(), ax=ax)
        ax.invert_yaxis()
        ax.set_title(f"# observations (log-scale)")
        ax.set_xlabel(f"Precipitations in {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations in {stations[subs[2]].name} (mm/h)")

        df = pd.DataFrame({'s2':s2s3_t_out[:,0],
                           's3':s2s3_t_out[:,1],
                           'p':cond_prob_t_out})

        pt = pd.pivot_table(df, values="p",index=["s2"], columns=["s3"], fill_value=np.nan, aggfunc='sum')
        pt2 = np.array(pt)

        # Clear wher we don't have enough observations
        OBS_THRESH=5
        m = np.array(grid_nb_obs) < OBS_THRESH
        pt2[m] = np.nan

        ax = axes[1]
        sns.heatmap(pt2, square=True, cmap="viridis", ax=ax)
        rng = range(0,len(pt.columns),2)
        ax.set_xticks(rng)
        ax.set_xticklabels(pt.columns[rng])
        rng = range(0,len(pt.index),2)
        ax.set_yticks(rng)
        ax.set_yticklabels(pt.index[rng])
        ax.invert_yaxis()
        ax.set_xlabel(f"Precipitations in {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations in {stations[subs[2]].name} (mm/h)")
        ax.set_title(f"P(some rain in {stations[subs[0]].name} given {stations[subs[1]].name} & {stations[subs[2]].name}) \nfor {OBS_THRESH} obs. at least. P(all) = 1 !")

        plt.savefig(f"two_cond_probabilities.pdf", format="pdf", bbox_inches="tight", dpi=600)
        plt.show()

    # ------------------------------------------------

    if False:
        fig = plt.figure()
        ax = fig.add_subplot(projection='3d')
        cs = ax.scatter(z[m,1], z[m,2], z[m,0], c=z[m,3]) #, short_cond_prob)
        ax.set_xlabel(f"Precipitations in {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations in {stations[subs[2]].name} (mm/h)")
        ax.set_zlabel(f"Precipitations in {stations[subs[0]].name} (mm/h)")
        ax.set_title(f"P(mm precipitations in {stations[subs[0]].name} given {stations[subs[1]].name} & {stations[subs[2]].name}) at $t$")
        fig.colorbar(cs, format='%.0e')
        plt.show()

    # ------------------------------------------------

    if False:
        # Keep data for S2 and S3
        z, inv_ndx = np.unique(all_data[:,1:3], axis=0, return_inverse=True)
        # Aggregate over time
        summed = npg.aggregate(inv_ndx, all_data[:,3], func='sum', fill_value=0)

        fig = plt.figure()
        ax = fig.add_subplot()
        #m = (z[:,0] < 5) & (z[:,1] < 5) & (z[:,0] > 0.1) & (z[:,1] > 0.1)
        m = (z[:,0] <= LIM) & (z[:,1] <= LIM) & (z[:,0] + z[:,1] >= LOW)
        cs = ax.scatter(z[m,0], z[m,1], c=summed[m]) #, short_cond_prob)
        ax.set_xlabel(f"Precipitations at {stations[subs[1]].name} (mm/h)")
        ax.set_ylabel(f"Precipitations at {stations[subs[2]].name} (mm/h)")
        ax.set_title(f"P(some precipitations in {stations[subs[0]].name} given {stations[subs[1]].name} & {stations[subs[2]].name}) at $t$")
        fig.colorbar(cs, format='%.0e')
        plt.show()

    exit()


    # We now remove redeundatn data for the plotting (else
    # there's too much to draw)

    _, short_ndx = np.unique(s1s2s3,axis=0,return_index=True)

    short_cond_prob = cond_prob[short_ndx]
    _, u = np.unique(s2s3,axis=0,return_inverse=True)
    short_s2s3 = u[short_ndx]
    _, u = np.unique(s1,axis=0,return_inverse=True)
    short_s1 = u[short_ndx]

    print(f"unique s2s3:{short_s2s3.shape} s1:{short_s1.shape}, cond_prob:{short_cond_prob.shape}")
    fig = plt.figure()
    ax = fig.add_subplot(projection='3d')
    #ax.scatter(short_s1, short_s2s3, short_cond_prob)
    ax.scatter(s1s2s3[:,0], s1s2s3[:,1], s1s2s3[:,2]) #, short_cond_prob)
    plt.show()



    #print(np.unique(s2s3,axis=0,return_counts=True))

def fit_storm_iat(data, stations): # iat = interarrival time
    fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(16, 8))
    fit_storm_iat2(data, stations, 0.1, 1.0, 0.1, axes[0])
    fit_storm_iat2(data, stations, 1, 5.0, 0.5, axes[1])
    plt.savefig(f"t1_correlation_pmf.pdf", format="pdf", bbox_inches="tight", dpi=600)
    plt.show()

    # v = np.array(data[(data["StationId"] == ROBERTVILLE)]["mm"])
    # v = np.nan_to_num(v, copy=True, nan=0.0, posinf=None, neginf=None)

    from matplotlib import ticker, colors

    v = np.array(data[(data["StationId"] == ROBERTVILLE)]["mm"])
    #v = np.array(data["mm"])
    print(np.max(v))
    nb_nans = np.count_nonzero(np.isnan(v))
    print(nb_nans)

    def helper(shift, ex_zone, ax):
        ZOOM=10 # used to make fractional rain measures integer (+ around operation).

        XEX_MIN = 1
        XEX_MAX = ex_zone*ZOOM
        YEX_MIN = 0
        YEX_MAX = ex_zone*ZOOM
        TICKS=10

        r = np.round(v * ZOOM).astype(int)

        # Build the grid of probabilities

        g = np.zeros( (np.max(r)+1, np.max(r)+1) )
        before, after = r[0:-shift], r[shift:]
        for i in range(len(r)-shift):
            # before = X axis; y axis = after.
            g[after[i],before[i]] += 1


        # Turn to cumulated probabilities ('cos it's more readable on the graph)
        for i in range(np.max(r)+1):
            g[:,i] = np.cumsum(g[:,i]) / (np.sum(g[:,i]) or 1)
            # Tried to display probabilities as they are
            # but due to color precision, one can't see enough
            # details. So don't try to do better than cumsum,
            # it's clearly the best.
            #g[:,i] = np.log(1+g[:,i] / (np.sum(g[:,i]) or 1))


        # Zooming on the most interesting part
        # extract = np.log(1 + g[0:5,0:100])
        #extract = g[0:5,0:100]

        extract = g[YEX_MIN:YEX_MAX, XEX_MIN:XEX_MAX]
        print(extract)

        # cs = ax.contourf( g[0:100,0:100], locator=ticker.LogLocator(), levels=20)
        # cs = ax.contourf( g[0:100,0:100], norm=colors.LogNorm(), levels=np.arange(0,1,0.1))
        vmin, vmax = np.min(extract), np.max(extract)
        print(f"vmin={vmin} vmax={vmax}")
        cs = ax.contourf( extract, levels=np.linspace(0,1,20+1)) #linspace go from 0 to 1 inclusive (not like arange)

        ax.set_title(f"{shift} hour delay")
        ax.set_xlabel("Rain (mm) at $t$")
        ax.set_ylabel(f"Rain (mm) at $t+{shift}$")

        t = np.arange(XEX_MIN,XEX_MAX,((XEX_MAX-XEX_MIN)//TICKS or 1))
        ax.set_xticks(t, labels=t/ZOOM)
        t = np.arange(YEX_MIN,YEX_MAX,((YEX_MAX-YEX_MIN)//TICKS or 1))
        ax.set_yticks(t, labels=t/ZOOM)

        return cs

    for region in (3,20):
        fig, axes = plt.subplots(nrows=1, ncols=2)
        fig.suptitle("P(mm < y at $t+1$ or $t+24$ | rain=x mm at $t$)")
        helper(1, region, axes[0])
        cs = helper(24, region, axes[1])
        fig.colorbar(cs)
        plt.savefig(f"t1_correlation_{region}mm.pdf", format="pdf", bbox_inches="tight")
        plt.show()


    exit()



    # Hours -------------------------------------------------------------------
    v = np.array(data["mm"])
    v = np.nan_to_num(v, copy=True, nan=0.0, posinf=None, neginf=None)

    all_stations_durations = np.array([])
    all_stations_iat = np.array([])
    for sid in stations:
        v = np.array(data[(data["StationId"] == sid)]["mm"])
        nzeros = non_zero_runs(v)
        storm_durations = nzeros[:,1] - nzeros[:,0]
        if max(storm_durations) > 100:
            print(f"Skipping {stations[sid]} because data looks weird")
            print(np.histogram(storm_durations))
            continue
        all_stations_durations = np.hstack( (all_stations_durations, storm_durations ))

        zeros = zero_runs(v)
        iat = zeros[:,1] - zeros[:,0]
        all_stations_iat = np.hstack( (all_stations_iat, iat ))

    print(all_stations_durations.shape)
    array_to_file(all_stations_durations, "all_stations_durations.txt")
    array_to_file(all_stations_iat, "all_stations_iat.txt")

    # Every stations Days --------------------------------------------------------------------
    v = np.array(data["mm"])
    days = np.reshape(v, (-1,24)) # One row == 1 day
    mm_per_day = days.sum(axis=1)
    array_to_file(mm_per_day,"all_stations_rain_per_day.txt")



    exit()
    # Individual stations Days --------------------------------------------------------------------
    v = np.array(data[(data["StationId"] == ROBERTVILLE)]["mm"])

    days = np.reshape(v, (-1,24)) # One row == 1 day
    mm_per_day_org = days.sum(axis=1)
    array_to_file(mm_per_day_org,"robertville_rain_per_days.txt")


    a = mm_per_day_org.copy()
    zeros = zero_runs(a)
    no_storm = zeros[:,1] - zeros[:,0]
    array_to_file(no_storm,"robertville_iat_days.txt")

    a = mm_per_day_org.copy()
    a[a < 1] = 0
    zeros = zero_runs(a)
    no_storm = zeros[:,1] - zeros[:,0]
    array_to_file(no_storm,"robertville_iat_days_filtered.txt")

    nzeros = non_zero_runs(v)
    storm_durations = nzeros[:,1] - nzeros[:,0]
    array_to_file(storm_durations,"robertville_storm_durations_hours.txt")

    # from distfit import distfit
    # dist = distfit(todf=True)
    # dist.fit_transform(storm_durations)
    # dist.plot()

    print("-"*80)

    zeros = zero_runs(v)
    no_storm = zeros[:,1] - zeros[:,0]

    #print(list(no_storm))

    # Val = intearrival time
    # counts = how many time we see the given interraival time
    val, counts = np.unique(no_storm, return_counts=True)

    #dist = stats.expon # loc, scale
    #params = dist.fit(no_storm)
    #fitted_dist = dist(*params)

    dist = stats.gamma # shape, loc, scale
    # The gamma fit with default fitter doesn't work
    # at all. Tried for both methods MLE & MM.
    # Note: in scipy's doc, they say the fit is a local
    # minimum, not a global one.
    # params = dist.fit(no_storm)

    # Manual fit gives much better result than automatic fit
    params = dist.fit(no_storm,method="MM")
    print(params)
    gamma_params = (0.8, 1, 4) # shape < 1 'cos we want exponential like. loc = 1 'cos our counts start at 1. scale=?
    params = dist.fit(no_storm, gamma_params[0], loc=gamma_params[1], scale=gamma_params[2])


    fitted_dist = dist(*params)

    plt.plot(val, counts/np.sum(counts), label="Interarrival time (h)")
    plt.plot(val, fitted_dist.pdf(val))
    plt.show()


def find_storms(data, stations):
    #v = np.array(data[(data["StationId"] == ROBERTVILLE)]["mm"])
    v = np.array(data["mm"])

    # Compute the precipitations per storm per day

    assert v.shape[0] % 24 == 0, "Every day must have 24 hours"

    days = np.reshape(v, (-1,24)) # One row == 1 day

    mm_per_day = days.sum(axis=1)
    # & (mm_per_day < 50)
    plt.hist(mm_per_day[(mm_per_day >= 1) ], bins=30)
    plt.title("Precipitations per day (at least 1 mm)") # , cut at 50 mm
    plt.xlabel('Precipitation (mm)')
    plt.ylabel('Frequency')
    plt.show()


    # Compute the storm duration
    print(f"filtered data shape {v.shape}")
    nzeros = non_zero_runs(v)
    storm_durations = nzeros[:,1] - nzeros[:,0]

    # -------------------------------------------------------------------------
    # Compute the total precipitation per storm

    # A storm = one ore more contiguous hours with rain
    # This may be too detailed as we end up having a
    # lot of 1 hour storms

    preci = []

    # iterate over each storm
    for i in range(nzeros.shape[0]):
        # p = how much precipitation in that storm
        p = np.sum(v[nzeros[i,0]:nzeros[i,1]])
        preci.append(p)
    p = np.array(preci)
    #plt.hist(p[(p > 10) & (p < 50)])
    plt.hist(p)
    plt.xlabel('Precipitation per storm (mm)')
    plt.ylabel('Frequency')
    plt.show()

    # -------------------------------------------------------------------------
    # Compute durations of storms and interarrival time

    # Compute the inter-storm time
    zeros = zero_runs(v)
    no_storm = zeros[:,1] - zeros[:,0]
    val, counts = np.unique(no_storm, return_counts=True)
    print(f"Min IAT == {min(val)}")
    plt.xlim(min(val), max(val))
    plt.xticks([1] + list(range(10,max(val)+1,10))) # lettre@plateforme-wallonne-giec
    plt.plot(val, counts, label="Interarrival time")

    # Storm durations
    val, counts = np.unique(storm_durations, return_counts=True)
    print(f"Min duration == {min(val)}")
    plt.xlim(min(val), max(val))
    plt.xticks([1] + list(range(10,max(val)+1,10)))
    plt.plot(val, counts, label="Storm duration")

    plt.title('Storms Statistics')
    plt.xlabel('Hours')
    plt.ylabel('Frequency')
    plt.legend()
    plt.show()

    # -------------------------------------------------------------------------
    # Compute the storm duration versus precipitation
    preci = []
    dura = []
    big_i = []
    for i in range(nzeros.shape[0]):
        j = nzeros[i,0]
        d = nzeros[i,1] - nzeros[i,0]
        p = np.sum(v[nzeros[i,0]:nzeros[i,1]])
        dura.append(d)
        preci.append(p)

        if p > 100 and d > 50:
            big_i.append((j, d, p))


    for i,d,p in big_i:
        row = data.iloc[i]
        plt.annotate(f"{stations[row.StationId].name} {dmy(row.Date)}", (d,p))

    plt.scatter(dura,preci,marker=".")
    plt.title(f"{nzeros.shape[0]} storms\n(all data)")
    plt.xlabel('Duration (hours)')
    plt.ylabel('Precipitation (mm)')
    plt.show()

    # -------------------------------------------------------------------------
    # Compute the storm duration versus precipitation density (mm/duration)

    d = np.hstack(
        (np.array(dura).reshape(-1,1),
        (np.array(preci) / np.array(dura)).reshape(-1,1)))
    print(d.shape)
    sdata, cnt = np.unique(d, axis=0, return_counts=True)
    print(sdata.shape)

    # FIXME Wrong ! The way I plot grid leads to the labels
    # of the y axis being wrong : some y-value may have been
    # removed from the pivot table
    df = pd.DataFrame({'d':sdata[:,0],
                       'p': np.round(sdata[:,1]),
                       'f':cnt})
    grid = pd.pivot_table(df, values='f',index=["p"], columns=["d"], fill_value=0, aggfunc=sum)
    print(grid.shape)


    fig = plt.figure()
    ax = fig.add_subplot()

    #cs = ax.contourf( grid, levels=np.linspace(0,1,20+1)) #linspace go from 0 to 1 inclusive (not like arange)
    sns.heatmap(grid, cmap="viridis", square=True, norm=LogNorm(), ax=ax)
    ax.invert_yaxis()
    rng = range(0,len(grid.index),2)
    ax.set_yticks(rng)
    ax.set_yticklabels(grid.index[rng])


    # cs = ax.scatter(sdata[:,0],
    #             sdata[:,1],
    #             c=cnt, cmap="viridis", marker='.', norm=LogNorm())
    # fig.colorbar(cs, format='%.0e')
    ax.set_title("Frequency vs Intensity vs Duration\n(all data)")
    ax.set_xlabel('Duration (h)')
    ax.set_ylabel('Intensity (mm/h)')
    plt.show()

def precipitation_per_month(data, stations):

    p_tot = np.zeros((20,12))

    # Using Pandas in a more vectorized way makes
    # thing 100x faster !

    year_min = data["Date"].min().year

    dm = [d.month for d in data["Date"]]
    dy = [d.year - year_min for d in data["Date"]]
    dp = np.nan_to_num(data["mm"], copy=True, nan=0.0, posinf=None, neginf=None)

    for y,m,p in tqdm(zip(dy,dm,dp), total=data.shape[0]):
        p_tot[y,m-1] += p

    for i in range(p_tot.shape[0]):
        plt.scatter(x=range(12), y=p_tot[i,:])
    plt.legend()
    plt.show()


def precipitation_cumulative(data, stations):
    # mm = filter_by(data, BATTICE, "1/1/2020", "31/12/2020")
    mm = data["mm"]
    values, bins = np.histogram(mm[mm>=0], bins=100)
    cs = np.cumsum(values)
    plt.plot(bins[:-1], cs / np.max(cs))
    plt.xlabel("Precipitations (mm/h)")
    plt.ylabel("P(mm/h $\leq$ x)")
    plt.title("CDF of precipitations per hour")
    plt.show()


def test_emd(data, stations):
    import emd


    # https://emd.readthedocs.io/en/stable/

    sample_rate = 24 # times a day
    sdata = data[data.StationId == BATTICE]
    y = sdata.sort_values(by="Date")['mm'].to_numpy()
    y = np.nan_to_num(y, copy=True, nan=0.0, posinf=None, neginf=None)
    print(y.shape)

    suby = y[0:10000]


    from scipy.optimize import minimize

    a = b = c = 1
    z_m = 0.01
    y0 = 0

    z_i = suby[suby >= z_m]


    plt.hist(z_i)
    plt.show()

    # Our latent varaible, normally dsitributed (mean=0, sigma²=1).
    y_i = np.random.normal(size=len(z_i))

    rg = z_m + b*(np.exp(a*((y_i - y0)**c)) - 1)
    plt.hist(rg)
    plt.show()


    # plt.plot(z_i)
    # plt.show()

    def log_likelihood( params):
        a,b,c = params[0],params[1],params[2],

        phi_inv = ((1/a) * np.log( (1/b)*(z_i - z_m) + 1))**(1/c) + y0

        # Maximizing f == minimizing -f
        return - np.sum(
            - ((1/phi_inv)**2)/2 \
            - 1/c*np.log(a) - np.log(c) \
            - np.log(z_i - z_m + b) \
            + (1/c - 1) * np.log(np.log((z_i - z_m)/b + 1)))

    # Pour que la log likelihood fonctionne, il faut
    # np.log( (1/b)*(z_i - z_m) + 1)) OK => z_i >= z_m : ok par csontrustciotn de zm
    # ((1/a) * np.log( (1/b)*(z_i - z_m) + 1)) > 0 car il est à l'exposant 1/c
    # donc sign(np.log(...))*sign(1/a) doit être positif

    lik_model = minimize(
        log_likelihood,
        np.array([1,1,1]),
        bounds = [(0.0001,100),(0.0001,100),(0.0001,100)],
        method='L-BFGS-B')
    print(lik_model)

    imf = emd.sift.sift(y,imf_opts={'sd_thresh': 1000})
    emd.plotting.plot_imfs(imf[:1000,:], cmap=True, scale_y=True)
    plt.show()



#def intensity_duration_frequency(data, stations):


def plot_correlation(data, stations):

    sid1 = ROBERTVILLE
    sid2 = NASSOGNE
    s1 = filter_by(data, sid1, "1/2007", "1/2022")
    s2 = filter_by(data, sid2, "1/2007", "1/2022")
    a = np.hstack( (s1,s2) )
    a = np.unique(a, axis=0) # Make charts faster
    print(a)
    plt.scatter(a[:,0],a[:,1],)
    plt.xlabel(f"Precipitations at {stations[sid1].name} (mm/h)")
    plt.ylabel(f"Precipitations at {stations[sid2].name} (mm/h)")
    plt.title("Correlation of Precipitations")
    plt.show()





def make_idf_curves(data, stations, fpath = None, axs=None, sid=None, return_periods=None):
    begin, end = f"1/1/2000",f"31/12/2022"
    storms, _ = compute_storms(filter_by(data, sid, begin=begin, end=end, col="all"))
    storms = storms[ (storms['Duration'] <= 15) & (storms['mm'] <= 15)]

    if fpath is not None:
        begin, end = np.min(storms["Date"]), np.max(storms["Date"])

        storms = pd.read_csv(fpath, header=0)
        storms = storms[ (storms['Duration'] <= 15) & (storms['mm'] <= 15)]

        storms['Duration'] = np.floor(storms['Duration'])

        print(storms)

        print(len(storms))
        nb_days = (end - begin).days
        print(nb_days)
        days_ofs = np.round(np.linspace(0, (end - begin).days, len(storms)))
        print(days_ofs.shape)

        print(f"Min:{np.min(storms['Duration']):.1f}h {np.min(storms['mm']):.1f}mm")
        dates = [begin + timedelta(days=int(day)) for day in days_ofs]
        storms.insert(1, 'Date', dates)



    period = np.max(storms["Date"]).year - np.min(storms["Date"]).year + 1

    if return_periods is None:
        RETURN_PERIODS = list(range(1,period+1))
    else:
        RETURN_PERIODS = return_periods

    print(f"Period = {period} years, {len(storms)} storms")
    all_intensities = storms["mm"]
    #all_intensities = storms["mm"] / storms["Duration"]
    all_durations = storms["Duration"]

    # Build the IDF curves, one duration at a time

    DURATIONS = list(range(1,40+1))
    idf = np.zeros((period, len(DURATIONS)))

    year_range = range(np.min(storms["Date"]).year,
                       np.max(storms["Date"]).year+1)

    for duration in DURATIONS:
        # We build the maximum intensities on each YEAR

        duration_filter = (storms["Duration"] >= duration) & (storms["Duration"] < duration+1)

        imax = []
        for year in year_range:
            b, e = human_date(f"1/1/{year}"), human_date(f"1/1/{year+1}")
            date_filter = (b <= storms["Date"]) & (storms["Date"] < e)

            # All the storms of the desired duration in the current year.
            # There might be none of them...
            a_year_of_storms = storms[ duration_filter & date_filter]

            if len(a_year_of_storms) > 0:
                # If we have no record, then it rained zilch.
                assert( np.sum(a_year_of_storms['mm'].isna()) == 0)
                imax.append(np.max(a_year_of_storms['mm']))

        print(f"Duration {duration} for years -> ", ",".join([f"{x:.1f}" for x in imax]))

        # Smallest intensities first
        imax = np.sort(np.array(imax))

        # # Pick the maximum intensities for the given duration; we take 'period' out of them
        # # The period is in years.
        # imax = np.sort(all_intensities[all_durations == duration])

        if len(imax) < period:
            imax = np.hstack([np.zeros(period - len(imax)), imax])
        else:
            imax = imax[len(imax)-period:len(imax)]

        print(f"{duration} -> ", imax)

        # ranked from lowest intensities (more likely, smaller return period) to highest.

        idf[:, duration-1] = imax

    if axs == None:
        fig, axs = plt.subplots(1,1,figsize=(6,6))

    for ndx, return_period in enumerate(RETURN_PERIODS):
        l = idf[return_period-1, :] / np.array(DURATIONS)
        l[l==0] = np.nan
        axs.plot(DURATIONS,
                 l,
                 c=cm.viridis(ndx/len(RETURN_PERIODS)),
                 label=f"{return_period}")

    if sid is None:
        s = "All stations"
    else:
        s = stations[sid].name

    if fpath:
        s = s+" "+fpath

    axs.set_title(f"Empirical IDF curves - {s}")
    axs.set_xlabel("Duration (h)")
    axs.set_ylabel("Intensity over rainstorm (mm/h)")
    axs.legend(title="Return Period\n(years)")

    # Simulation
    """

    RETURN_PERIOD = 3 (where maximum value is 19)

    is interpreted as :

    The event with return period == 3 years has a probability=3/19
    of occuring
    """


def compute_storms_avg_maxima(data,
                              noise_level=0.1,
                              min_precipitations_per_storm=2,
                              include_sids=False):
    # Returns two data frames:
    # - one with the storms
    # - one with the inter arrival time between storms.

    precipitations = []
    durations = []
    sids = []
    dates = []
    inter_storm_durations = []
    inter_storm_dates = []
    inter_storm_sids = []
    pp_avg_max = dict()

    #print(f"{len(data['StationId'].unique())} different stations")

    # We compute station by station because recording
    # timelines are at the station level.
    for sid in tqdm(data['StationId'].unique()):

        station_data = data[ data['StationId'] == sid]
        assert len(station_data) > 0, "No storm to process ?"
        v = np.array(station_data["mm"])
        vd = np.array(station_data["Date"])

        noise = v < noise_level
        v[noise] = 0

        # We rely on the fact that records are done every hours
        # So one index position equals one hour.
        nzeros = non_zero_runs(v)

        last_storm_end = last_storm_end_date = None
        for i in range(nzeros.shape[0]):
            # nzeros[i,0] = *index* of a run of non zero precipitation
            # We rely on the fact that recordings happen on *each* *hour* (both are important here)
            # so an index is expressed in hours.
            h_storm_begin = nzeros[i,0]
            h_storm_end = nzeros[i,1]

            # Rain event duration (in hour)
            # Float is better suited to further analysis
            d = float(h_storm_end - h_storm_begin)

            # Total precipitation during a rain event
            p = np.sum(v[nzeros[i,0]:nzeros[i,1]])

            # keep only significant rain events
            # i.e. those with enough rainfall.
            if p >= min_precipitations_per_storm: # Clean up some bad data,
                durations.append(d)
                precipitations.append(p)
                dates.append(vd[nzeros[i,0]])
                sids.append(sid)

                if last_storm_end is not None:
                    iat = h_storm_begin - last_storm_end
                    inter_storm_durations.append(iat)
                    inter_storm_dates.append(last_storm_end_date)
                    inter_storm_sids.append(sid)

                last_storm_end = h_storm_end
                last_storm_end_date = vd[nzeros[i,1]]

                # We also compute the averages over several time steps

                storm_steps = v[nzeros[i,0]:nzeros[i,1]]
                for i in range(1,int(d)+1):
                    m = np.max(np.convolve(storm_steps, np.ones(i) * 1/i))

                    if i not in pp_avg_max:
                        pp_avg_max[i] = []
                    pp_avg_max[i].append(float(m))


    for k,v in pp_avg_max.items():
        plt.scatter( [k]*len(v), v, marker=".")
    plt.xlabel("Duration d")
    plt.xlabel("Max of average hourly precipitation for d")
    plt.show()

    fig, axs = plt.subplots(5,5)
    axs = axs.ravel()
    for i in range(5*5):
        if i == 0:
            axs[i].set_xlabel("Max of average pp (mm)")
            axs[i].set_ylabel("Frequency")
        axs[i].hist(pp_avg_max[i+1],bins=20,label=f"Duration {i+1}")
        axs[i].legend()

    plt.show()

    # The roundings are here to fix small computation errors
    # that occur during processing (FIXME : find the origin of these errors)
    # The rounding reproduces the rounding in the actual data.
    d = {
        "Date": dates,
        "Duration": np.round(durations),
        "mm": np.round(precipitations,1), # Quantity of water in a given storm
    }
    if include_sids:
        d["StationId"] = sids
    storms = pd.DataFrame(d)

    d = {
        "Date": inter_storm_dates,
        "Duration": np.round(inter_storm_durations)
    }
    if include_sids:
        d["StationId"] = inter_storm_sids
    inter_storms = pd.DataFrame(d)

    return storms, inter_storms


def compute_storms_separation_chart(data, fpath, title):
    df = {"noise_level":[],
          "min_precipitations_per_storm":[],
          "nb_rain_episodes" : []}
    print(f"{len(data)} records")

    for noise_level in tqdm([0.2,0.3,0.5,0.7,1]):
        for min_precipitations_per_storm in [1,2,3,4,6,8,10,15,20]:
            df["noise_level"].append(noise_level)
            df["min_precipitations_per_storm"].append(min_precipitations_per_storm)
            storms, _ = compute_storms(data, noise_level, min_precipitations_per_storm)
            df["nb_rain_episodes"].append(len(storms))

    pd.DataFrame(df).to_csv(fpath, index=False)

    df = pd.read_csv(fpath)
    pt = pd.pivot_table(
        df, values="nb_rain_episodes", index=["noise_level"], columns=["min_precipitations_per_storm"]
    )
    ax = sns.heatmap(pt, annot=True, fmt="3g")
    ax.invert_yaxis()
    ax.set_title(f"Number of storms ~ Simple storms separation params\n{title}")
    plt.savefig("rain_episode_separation_simple.png")
    plt.show()

def compute_noise_analysis(all_data, STATIONS, noise_level=0.1, min_precipitations_per_storm=2):
    noises = []
    in_storms = []
    good_sids = []
    print(all_data)
    for sid in STATIONS.keys():
        data = all_data[all_data['StationId'] == sid]
        if len(data) == 0:
            continue
        total = data['mm'].sum()
        good_sids.append(sid)
        storm, _ = compute_storms(
            data,
            noise_level=noise_level,
            min_precipitations_per_storm=min_precipitations_per_storm,
            include_sids=True)
        print(f"{sid} {type(sid)}: {len(storm)} storms, len hyeto:{len(data)} max hyeto: {data['mm'].max()}")
        in_storms.append( storm['mm'].sum() / total )
        noise = data[data['mm'] < noise_level]['mm'].sum()
        noises.append(noise/total)

    noises = np.array(noises)
    in_storms = np.array(in_storms)

    labels = range(len(good_sids))
    plt.bar(labels,
            np.ones(len(noises)) - (in_storms + noises),
            bottom=noises + in_storms,
            label=f"Storms ≤ {min_precipitations_per_storm}mm ({100*np.mean(np.ones(len(noises)) - (in_storms + noises)):.1f}%)")
    plt.bar(labels, in_storms, bottom=noises, label=f"Regular torms")
    plt.bar(labels, noises, label=f"< {noise_level}mm ({100*np.mean(noises):.1f}%)")

    # plt.scatter(range(len(ALL_SIDS)), noises)
    # plt.scatter(range(len(ALL_SIDS)), in_storms)
    # plt.xticks(range(len(ALL_SIDS)), [str(STATIONS[sid])[:4] for sid in ALL_SIDS])
    plt.yticks(np.arange(5)/4, [f"{int(100*p/4)}%" for p in range(5)])
    plt.title(f"{len(good_sids)} stations")
    plt.grid()
    plt.legend()
    plt.savefig(f"noise_overview_{noise_level}.png")
    plt.show()


import re
NUMBER_RE = re.compile("([0-9]+)")

def parse_irm_duration(s, as_hours=True):
    n = int(NUMBER_RE.match(s).group(0))

    if not as_hours:
        if "min" in s:
            return timedelta(minutes=n)
        elif "h" in s:
            return timedelta(hours=n)
        elif "d" in s:
            return timedelta(days=n)
    else:
        if "min" in s:
            return n/60
        elif "h" in s:
            return n
        elif "d" in s:
            return n*24



if __name__ == '__main__':

    # df = pd.DataFrame({
    #     'x':[1.5,2,3,1.5],
    #     'y':[1,3,2,1],
    #     'v':[9,8,7,6]})
    # print(pd.pivot_table(df, values='v', index=['y'], columns=['x'], fill_value=0, aggfunc='sum')/2)


    args = parser.parse_args()

    STATIONS = load_station_data(f"{DATAPATH}/stations.csv")

    slist = [s.name for s in STATIONS.values()]
    print(f"Stations metadata for: {', '.join(sorted(slist))}")

    if args.sid:
        selected_station = None
        for k,v in STATIONS.items():
            if v.name.startswith(args.sid[0]):
                print(f"Selected station is {v}")
                selected_station = v
                break
        if selected_station is None:
            print(f"Can't find station {args.sid[0]}. You can try {', '.join([s.name for s in STATIONS.values()])}")
            exit()
    else:
        print("All stations selected")
        selected_station = None

    print(f"Default noise level = {args.noise}")

    series_file = f"{DATAPATH}/series.csv"
    if args.import_data or not os.path.exists(series_file):
        if args.full_series:
            data = load_series(extended=True)
            #data.to_csv(series_file)
            series_to_numpy(data, prefix="all")
        else:
            stations, data = load_series()
            #data.to_csv(series_file)
            series_to_numpy(data)


    # Panda code to read from CSV
    # start_time = time.perf_counter()
    # data = pd.read_csv(series_file, parse_dates=[3])
    # print(f"Loaded {series_file} in {time.perf_counter() - start_time:.1f} seconds")

    start_time = time.perf_counter()

    data = load_data(DATAPATH,  args.full_series)

    print(f"Loaded NumPy data in {time.perf_counter() - start_time:.1f} seconds")

    if not args.analyze_quality:
        broad_select_bang(data, STATIONS) # Full of side effects !

    slist = [STATIONS[sid].name for sid in data["StationId"].unique()]
    print(f"Stations in the data set {', '.join(sorted(slist))}")

    #exit()

    if args.map:
        plot_cities(data, STATIONS)
    elif args.plot_all_series:
        plot_all_series(data, STATIONS)
    elif args.plot_some_series is not None:
        a = args.plot_some_series

        if len(a) >= 1:
            start = human_date(a[0])
        else:
            start = datetime(2020,1,1)

        if len(a) == 2:
            end = human_date(a[1])
        else:
            end = datetime.today()

        assert start < end

        #plot_some_series(data, STATIONS, [RACHAMPS, WAREMME, MODAVE], start, end)
        #plot_some_series(data, STATIONS, [GEMMENICH, WAREMME, LANAYE], start, end)
        plot_some_series(data, STATIONS, [MODAVE, WAREMME, TERNELL], start, end)

    elif args.cumulative_series:
        # plot_cumulative(data, STATIONS, [TAILLE, RACHAMPS, ORTHO])
        plot_cumulative(data, STATIONS, [RACHAMPS, WAREMME, MODAVE]) # , LANAYE, RACHAMPS,
    elif args.animate_cities:
        make_anim(data, STATIONS)
    elif args.pearson:
        compute_pearson_correlation(data, STATIONS)
    elif args.rain_distribution:
        rain_distribution(data, STATIONS)
    elif args.analyze_quality:
        analyze_quality(data, STATIONS)
    elif args.find_storms:
        find_storms(data, STATIONS)
    elif args.precipitation_per_month:
        precipitation_per_month(data, STATIONS)
    elif args.precipitation_cumulative:
        precipitation_cumulative(data, STATIONS)
    elif args.test_emd:
        test_emd(data, STATIONS)
    elif args.fit_storm_iat:
        fit_storm_iat(data, STATIONS)
    elif args.fit_proba:
        fit_proba(data, STATIONS)
    elif args.plot_correlation:
        plot_correlation(data, STATIONS)
    elif args.make_idf_curves:

        # begin, end = f"1/1/2000",f"31/12/2022"
        # storms, _ = compute_storms(filter_by(data, None, begin=begin, end=end, col="all"))
        # idf = compute_idf_curves(storms, STATIONS, [1,5,10,20])
        # print(idf)

        # plt.plot(idf.columns, idf.loc[10])
        # plt.plot(idf.columns, idf.loc[20])
        # plt.show()
        # exit()

        if args.make_idf_curves == True:
            # We draw the IDF curves for our dataset
            make_idf_curves(data, STATIONS)
        else:
            # We draw the IDF curves for our dataset AND
            # for another dataset stored in a file
            fig, axs = plt.subplots(1,2,figsize=(6,6),sharey=True)
            if selected_station is not None:
                sid = selected_station.sid
            else:
                sid = None
            make_idf_curves(data, STATIONS, None, axs[0], sid=sid, return_periods=[2,5,10,15,20])
            axs[0].grid()
            # make_idf_curves(data, STATIONS, args.make_idf_curves, axs[1], sid=sid)
            # axs[1].grid()
        plt.show()

    elif args.bare_ts:
        if selected_station is not None:
            d = data[data["StationId"] == selected_station.sid]
            fpath = f"ts_{selected_station.name}.csv"
        else:
            d = data
            fpath = "ts_all.csv"

        d.to_csv(fpath, index=False)

    elif args.export_storms:
        if selected_station is not None:
            d = data[data["StationId"] == selected_station.sid]
            fpath = f"storms_{selected_station.name}.csv"
        else:
            d = data
            fpath = "all.csv"

        storms, inter_storms = compute_storms(
            d, noise_level=args.noise,
            min_precipitations_per_storm=1,
            include_sids=True,
            station_metadata=STATIONS)

        storms.to_csv(fpath, index=False)

        # I leave the noise inside because it will be needed
        # to build the noise model afterwards

        if selected_station is not None:
            fpath = f"time_series_{selected_station.name}.csv"
            d.to_csv(fpath, index=False)
            print(f"Exported {len(storms):n} storms in {fpath}")
        else:
            fpath = f"time_series_all.csv"
            d.to_csv(fpath, index=False)
            print(f"Exported {fpath}")

            for sid in storms["StationId"].unique():
                # Export hyetogram
                fpath = f"time_series_{STATIONS[sid].name}.csv"
                d[d["StationId"] == sid].to_csv(fpath, index=False)
                print(f"Exported {fpath}")

                # Export storms
                fpath = f"storms_{STATIONS[sid].name}.csv"
                s= storms[ storms["StationId"] == sid]
                s.to_csv(fpath, index=False)
                print(f"Exported {len(s):n} storms in {fpath}")



    elif args.max_avg_storms:
        if selected_station is not None:
            d = data[data["StationId"] == selected_station.sid]
            fpath = f"storms_{selected_station.name}.csv"
        else:
            d = data
            fpath = "all.csv"

        storms, inter_storms = compute_storms_avg_maxima(
            d,
            noise_level=0.1,
            min_precipitations_per_storm=2,
            include_sids=True)

    elif args.compute_storms_separation_chart:
        if selected_station is not None:
            print(f"Slecteing stations {selected_station.sid}")
            d = data[data["StationId"] == selected_station.sid]
            fpath = f"simple_storms_params_{selected_station.name}.csv"
            title = selected_station.name
        else:
            print(f"Slecteing all stations")
            d = data
            fpath = "simple_storms_params_all.csv"
            title = "All stations"
        compute_storms_separation_chart(d, fpath, title)

    elif args.noise_analysis:
        compute_noise_analysis(data, STATIONS, noise_level=args.noise, min_precipitations_per_storm=1)

    elif args.plot_irm_idf_simple:
        name = args.plot_irm_idf_simple

        returnlevel,standarddev,confintlow,confintup,montana = ins_read_excel(name=name)
        idf = returnlevel.transpose()
        idf.rename_axis("Return Period", axis="index", inplace=True)
        idf.columns = [parse_irm_duration(x) for x in idf.iloc[[0]].to_numpy()[0]]
        idf.drop(index="Duur / Durée", inplace=True)
        idf.index = idf.index.values.astype(int)
        idf = idf.div(idf.columns.values)

        print(f"Saving idf_irm_{name}.csv")
        idf.to_csv(f"idf_irm_{name}.csv")

        fig, axs = plt.subplots(1,1,figsize=(4,4), dpi=300)
        axs.grid()
        axs.set_title(f"Precipitation-density-frequency - {name}")
        plot_idf(idf, axs, skip_durations=3, plot_pp=True)
        fname = f"idf_irm_simple_{name}.png"
        plt.savefig(fname)
        print(f"Saved {fname}")


    elif args.plot_irm_idf:
        name = args.plot_irm_idf
        print(name)
        # h = data[data["StationId"] == BASTOGNE]
        # #h = h[ h["Date"] < date(20) ]
        # idf =  compute_irm_idf(
        #     pd.Series(data=h['mm'].to_numpy(),
        #               index=h['Date'].to_numpy()),
        #     bm_block_size = "182.625D",
        #     #durations = [1,2,3,4,5,6],
        #     plot_analysis=False)

        # fig, axs = plt.subplots(1,1,figsize=(6,6))
        # plot_idf(idf, axs, rp_to_years_factor=0.5)
        # plt.show()
        # #exit()

        returnlevel,standarddev,confintlow,confintup,montana = ins_read_excel(name=name)



        # returnlevel["Duration"] = returnlevel["Duur / Durée"].apply(parse_irm_duration)
        # returnlevel = returnlevel.set_index("Duration")

        idf = returnlevel.transpose()
        idf.rename_axis("Return Period", axis="index", inplace=True)
        idf.columns = [parse_irm_duration(x) for x in idf.iloc[[0]].to_numpy()[0]]
        idf.drop(index="Duur / Durée", inplace=True)
        idf.index = idf.index.values.astype(int)
        idf = idf.div(idf.columns.values)

        fig, axs = plt.subplots(1,2,figsize=(12,6), sharey=False)
        plot_idf(idf, axs[0], skip_durations=3, plot_pp=True)
        plot_idf(idf, axs[1], skip_durations=3, plot_pp=False)
        axs[0].set_title(f"Precipitation-density-frequency")
        axs[1].set_title(f"Intensity-density-frequency")
        axs[0].grid()
        axs[1].grid()
        plt.savefig("idf_irm_bastogne.pdf")

        fig, axs = plt.subplots(1,2,figsize=(12,6), sharey=True)
        plot_idf(idf, axs[0],
                 skip_durations=3, plot_pp=True)
        axs[0].set_title(f"IDF of IRM for {name} (xls)")
        axs[0].grid()

        # plot_idf(compute_simple_IDF(data[data["StationId"] == BASTOGNE]),
        #          axs[1])
        # axs[1].set_title(f"Simple IDF of Wallonie for {name}")
        # axs[1].grid()


        print("----------------------------------------------------------")
        h = data[data["StationId"] == BASTOGNE]
        print(h)
        print(h["mm"])
        print(h["mm"].max())
        print("----------------------------------------------------------")
        print(pd.Series(data=h['mm'].to_numpy(),
                        index=h['Date'].to_numpy()))
        idf, shapes =  compute_irm_idf(
            pd.Series(data=h['mm'].to_numpy(),
                      index=h['Date'].to_numpy()),
            return_periods = (np.array([2,5,10,15,20,25,30,40,50,75,100,200])*2).tolist(),
            bm_block_size = "182.625D",
            plot_analysis=False)

        plot_idf(idf, axs[1], rp_to_years_factor=0.5, plot_pp=True)

        for d, shape in shapes.items():
            x = IRM_DURATIONS.index(d)
            if shape > 0:
                axs[1].text(x,1,"+")
            elif shape < 0:
                axs[1].text(x,1,"-")
            else:
                axs[1].text(x,1,"0")

        axs[1].set_title(f"DGO data {name}, IRM computation")
        axs[1].grid()

        plt.savefig("compare_idf_irm_dgo_bastogne.pdf")
        plt.show()



    else:
        parser.print_help()
