library(lubridate)
library(rgl)
library(ggplot2)
library(viridis) # turbo palette
library(patchwork) # sum ggplots
source("raingen.R")

# FIXME Dezombify this
function() {
    fnames <- Sys.glob("hyeto_optim_h_*.csv")
    dmaps <- list()
    all_storms <- c()
    for(fname in fnames) {
        print(fname)
        hyeto <- read.table(fname,sep=",",header=TRUE)
        hyeto$Date <- ymd_hms(hyeto$Date)
        #print(head(hyeto))
        storms <- hyetogram_to_storms(hyeto)
        all_storms <- rbind(all_storms, storms)
        ## print(head(storms))
        dmap <- array(0,c(200,3000))
        for(sndx in seq(nrow(storms))) {
            storm <- storms[sndx,]
            print(sum(storms$mm*10 > 0))
            dmap[storm$Duration, storm$mm*10] <- dmap[storm$Duration, storm$mm*10] + 1
        }
        dmaps[[length(dmaps)+1]] <- dmap
        ## print(nrow(storms))
        ## print(sum(dmap))
        if(length(dmaps)> 10) {
            break
        }
    }

    # Group all dmap in one data cube
    cube <- array(0,c(length(dmaps),200,3000))
    for( i in seq(length(dmaps))) {
        cube[i,,] <- dmaps[[i]]
    }

    cube[cube == 0] <- NA # Mark pairs where we don't have any value
    cube_mn <- colMeans(cube,dims=1,na.rm=TRUE)
    cube_sd <- apply(cube, c(2,3), sd,na.rm=TRUE) # This produces a lot of NA's because many sd's are zero
    cube_centered <- ((colSums(cube,dims=1) - cube_mn) / cube_sd) ** 2

    z <- colSums(cube,dims=1,na.rm=TRUE)
    msk <- !is.na(z) & (z > 0)
    df <- as.data.frame(which(msk, arr.ind=TRUE))
    df$Mean <- z[msk]
    ggplot(df, aes(row,col,fill=Mean)) + geom_tile() # + xlim(0,10) + ylim(0,100)

    ## for(i in seq(length(dmaps))) {
    ##     #cube_centered[i,,] <- ((cube[i,,] - cube_mn) / cube_sd)**2
    ##     cube_centered[i,,] <- (cube[i,,] - cube_mn) / cube_sd
    ## }

    z <- cube_sd
    msk <- !is.na(z) & (z > 0)
    df <- as.data.frame(which(msk, arr.ind=TRUE))
    df$Mean <- z[msk]
    ggplot(df, aes(row,col,fill=Mean)) + geom_tile() + xlim(0,30) + ylim(0,300)

    z <- cube_mn
    msk <- !is.na(z) & (z > 0)
    df <- as.data.frame(which(msk, arr.ind=TRUE))
    df$Mean <- z[msk]
    p1 <- ggplot(df, aes(row,col,fill=Mean)) + geom_tile()
    z <- cube_centered
    msk <- !is.na(z) & (z > 0)
    df <- as.data.frame(which(msk, arr.ind=TRUE))
    df$Mean <- z[msk]
    p2 <- ggplot(df, aes(row,col,fill=Mean)) + geom_tile()
    p1+p2

    ggplot(all_storms, aes(row,col)) + stat_bin2d() #+ scale_y_continuous(trans='log10') + scale_x_continuous(trans='log10')


    ggplot(all_storms, aes(Duration,mm)) + stat_density2d()



    array(dmaps, c(200,200,3))
}

#######################################################################
read_storm_file <- function(filename) {

    if (!file.exists(filename)) {
        print(paste("!!! FAIL File doesn't exist", filename))
        return(NULL);
    }

    # This will fixe blank periods at the beginnignad end of the time series.
    storms <- read.table(filename,sep=",",header=TRUE)

    if(length(which(is.na(ymd_hms(head(storms$Date,50),quiet=TRUE)))) == 0) {
        storms$Date <- ymd_hms(storms$Date)
        print(paste("Loaded",filename,",",nrow(storms),"storms, YMD HMS format"))
    } else {
        storms$Date <- ymd(storms$Date)
        print(paste("Loaded",filename,",",nrow(storms),"storms, YMD format"))
    }

    return(storms)
}

read_all_simulations_storms_only <- function(shape) {
    # Read storm simulation (not hyeto simulation)
    all_sims <- lapply(TEST_STATIONS_NAMES, function(station_name) {
        sim_names <- Sys.glob(paste("simstorm_",station_name,"_",shape,"_*.csv",sep=""))
        l <- lapply(sim_names, function(fname) {read_storm_file(fname)})
        print(length(l))
        l
    })
    h <- hash(TEST_STATIONS_NAMES, all_sims)
    for( n in TEST_STATIONS_NAMES) {
        print(paste("Loaded",length(h[[n]]),"simulations for",n))
    }
    return(h)
}




shrink_vec <- function(a) {
    nz <- which(a$mm>0)
    a[head(nz,1):tail(nz,1),]
}

read_ts_for_idf <- function(filename) {
    if (!file.exists(filename)) {
        print(paste("!!! FAIL File doesn't exist", filename))
        return(NULL);
    }
    # This will fixe blank periods at the beginnignad end of the time series.
    full_ts_sim <- read.table(filename,sep=",",header=TRUE)
    #print(paste("Loaded",filename))
    full_ts_sim$Date <- ymd_hms(full_ts_sim$Date)
    nz <- which(full_ts_sim$mm > 0)
    full_ts_sim <- full_ts_sim[min(nz):max(nz),]
    print(paste("Read",filename,nrow(full_ts_sim),"rows"))
    return(full_ts_sim)
}

read_all_simulations_time_series <- function(shape) {
    # Read storm simulation (not hyeto simulation)
    all_sims <- lapply(TEST_STATIONS_NAMES, function(station_name) {
        sim_names <- Sys.glob(paste("hyeto_",station_name,"_",shape,"_*.csv",sep=""))
        l <- lapply(sim_names, function(fname) {read_ts_for_idf(fname)})
        l
    })
    h <- hash(TEST_STATIONS_NAMES, all_sims)
    for( n in TEST_STATIONS_NAMES) {
        print(paste("Loaded",length(h[[n]]),"simulations for",n))
    }
    return(h)
}

# Using the flat storm is enough since we're just looking at
# the storms, not the hyeto.
all_time_series <- read_all_simulations_time_series("flat")
all_storms <- read_all_simulations_storms_only("flat")


#observations <- extract_storms_hyetograms(c(OBSERVATIONS))
#######################################################
# PLOT IDF

compute_idf_projections <- function(full_ts,
                                    force_weibull=FALSE,
                                    block_size=6,
                                    ret_periods=c(2,5,10,15,20,25,30,40,50,75,100,200),
                                    pdf=TRUE) {
    durations <- c(1,2,3,6,12, c(1,2,3,4,5,7,10,15,20,25,30)*24)
    ret_levels <- c()
    for(dndx in seq(length(durations))) {
        duration <- durations[dndx]
        ## d2 <- data.frame(
        ##     Date=full_ts$Date,
        ##     mm=c(filter(full_ts$mm, rep(1,duration), method="convolution", circular=TRUE)),
        ##     # Blocks are half years (mon-1:0..11 =>
        ##     block=year(full_ts$Date) + 0.5*floor((month(full_ts$Date)-1)/6))
        ## maxima <- blockmaxxer(d2, blocks=d2$block, which="mm")$mm
        maxima <- convolve_and_bm(full_ts, duration, block_size)
        print(paste(dndx,"/",length(durations)))
        ## print(maxima)
        ## print(tail(sort(full_ts$mm),10))
        ## fit <- fevd( maxima, type = "GEV", method = "Bayesian") #Lmoments
        ## par <- list(shape=mean(fit$results[,"shape"]),
        ##             location=mean(fit$results[,"location"]),
        ##             scale=exp(mean(fit$results[,"log.scale"])))
        fit <-  fevd( maxima, type = "GEV", method = "Lmoments") #Lmoments
        #par <- fit$results$par
        par <- list(location=fit$results["location"],
                    shape=fit$results["shape"],
                    scale=fit$results["scale"])
        if (force_weibull) {
            o <- optim(
                c(par[["location"]], scale=par[["scale"]],
                  if(par[["shape"]]<0) {par[["shape"]]} else {-0.2}),
                function(par) {
                    #print(par)
                    r <- sum(log(devd(maxima,loc=par[1],scale=par[2],shape=par[3])))
                    #print(r)
                    if( r == -Inf || is.nan(r)) {
                        r <- -1e200
                    }
                    return(r)
                },
                method="L-BFGS-B",
                lower=c(   0,    0.1,    -2),
                upper=c(160, 60,    -0.00000000001),
                control = list(fnscale=-1))# fnscale for maximisation)
            par <- list(location=o$par[1],scale=o$par[2],shape=o$par[3])
        }
        ## print("--------------------------")
        ## print(c(fit$results$par))
        ## print(unlist(par))
        # Calculate return levels for extreme value distributions (EVDs).
        ret_lev <- rlevd(
            ret_periods*(12/block_size), # Convert back to years
            loc=par[["location"]], scale=par[["scale"]], shape=par[["shape"]])
        if (pdf == FALSE) {
            ret_lev <- ret_lev / duration
        }
        ret_levels <- rbind(ret_levels, ret_lev)
    }
    colnames(ret_levels) <- paste(ret_periods," years",sep="")
    rownames(ret_levels) <- pretty_hours(hours(durations))
    return(ret_levels)
}


compute_all_idf <- function(sim_file_name, shape, max_files=10, pdf=TRUE,
                            ret_periods=c(2,5,10,15,20,25,30,40,50,75,100,200)) {
    all_sim_idf <- list()
    fnames <- Sys.glob(paste("hyeto_",sim_file_name,"_",shape,"_*.csv",sep=""))
    for(fndx in seq(min(max_files,length(fnames)))) {
        print(fnames[fndx])
        full_ts_sim <- read_ts_for_idf(fnames[fndx])
        idf <- compute_idf_projections(full_ts_sim, pdf=pdf, ret_periods=ret_periods)
        all_sim_idf <- append(all_sim_idf, list(idf))
    }
    return(all_sim_idf)
}


plot_idf <- function( idf_matrix, make.plot=TRUE, pdf=TRUE, ylim=NA, ...) {
    palette <- turbo(ncol(idf_matrix))
    if(make.plot) {
        ylimit <- if(is.na(ylim)) {max(idf_matrix)} else {ylim}
        plot(1, type = "n", xlab = "Durations", xaxt="n", yaxt="n",
             xlim = c(1, nrow(idf_matrix)),
             ylim = c(1, ylimit),
             ...)
        xaxis <- seq(nrow(idf_matrix))
        axis(side = 1, at = xaxis,labels = rownames(idf_matrix))
        yaxis <- seq(0,ylimit,by=if(ylimit < 100) {10} else {50})
        axis(side = 2, at = yaxis,labels = yaxis)
        # Horizontal lines of the grid
        abline(h=yaxis, col="grey")
        # Vertical lines of the grid
        abline(v=xaxis, col="grey")
        # Draw legend /over/ the grid
        legend(if(pdf==TRUE){"topleft"}else{"topright"},
               legend=colnames(idf_matrix),
               col=palette,
               lty = 1)
    }
    # Return levels lines
    for(i in seq(ncol(idf_matrix))) {
        r <- c(t(idf_matrix[,i]))
        lines(r, col=palette[i])
    }
}

if(TRUE) {
    for(PDF in c(TRUE,FALSE)) {
    ylim <- if(PDF==TRUE){400}else{70}
    prefix <- if(PDF==TRUE){"pdf"}else{"idf"}
    ylab <- if(PDF==TRUE){"Total (mm)"}else{"Intensity (mm/h)"}
    #par(mfrow=c(1,3))
    for( irm_name in c("Bastogne","Waimes","Wavre")) {
        irm_idf <- read.csv(paste("idf_irm_",irm_name,".csv",sep=""),
                            header=TRUE,check.names=FALSE, row.names = 1)
        irm_idf <- irm_idf[,4:ncol(irm_idf)]
        if(PDF == TRUE) {
            irm_idf <- sweep( irm_idf,  MARGIN = 2, STATS = as.numeric(colnames(irm_idf)), "*")
        }
        colnames(irm_idf) <- pretty_hours(hours(as.numeric(colnames(irm_idf))))
        rownames(irm_idf) <- paste(rownames(irm_idf)," years",sep="")
        out_plot(paste("irm_",prefix,"_",irm_name,sep=""),
                 width=8,height=8, res=150)
        plot_idf(t(irm_idf),pdf=PDF,ylim=ylim,main=paste("IRM - ",irm_name, sep=""),ylab=ylab)
        dev.off()
    }
    N_SIM <- 20
    for(shape in c("triangle")) {
        for(sim_file_name in TEST_STATIONS_NAMES) { #, ROBERTVILLE, WAVRE)) {
            # Read original data
            observations <- extract_storms_hyetograms(string_to_station_id[[sim_file_name]])
            obs_idf <- compute_idf_projections(observations$orig_tseries, pdf=PDF)
            # Read simulations data
            all_sim_idf <- compute_all_idf(sim_file_name, shape, max_files=N_SIM, pdf=PDF)
            avg_idf <- Reduce("+",all_sim_idf) / length(all_sim_idf)
            # Draw plot
            out_plot(paste(prefix,"_sim_vs_obs_",sim_file_name,"_",shape,sep=""),
                     width=16,height=8, res=150)
            par(mfrow=c(1,2))
            plot_idf(obs_idf,pdf=PDF,ylim=ylim,main=paste("Observations - ",sim_file_name, sep=""),ylab=ylab)
            plot_idf(avg_idf,pdf=PDF,ylim=ylim,main=paste(sim_file_name,", ",shape," shape, ",N_SIM," sims average", sep=""),ylab=ylab)
            dev.off()
        }
    }
    } # loop PDF or IDF
}




sim_file_name <- "BASTOGNE"
shape <- "triangle"
N_SIM <- 10
all_sim_idf <- compute_all_idf(sim_file_name, shape, max_files=N_SIM)
avg_idf <- Reduce("+",all_sim_idf) / length(all_sim_idf)

observations <- extract_storms_hyetograms(BASTOGNE)

obs_idf <- compute_idf_projections(observations$orig_tseries, pdf=FALSE)
plot_idf(obs_idf,main=paste("Observations - ",sim_file_name, sep=""),ylab="Total mm")

par(mfrow=c(1,2))
plot_idf(obs_idf,ylim=350,main=paste("Observations - ",sim_file_name, sep=""),ylab="Total mm")
plot_idf(avg_idf,ylim=350,main=paste(sim_file_name,", ",shape," shape, ",N_SIM," sims average", sep=""),ylab="Total mm")



make_and_plot_idf <- function(full_ts, main, force_weibull=FALSE, plot_intensities=FALSE, block_size=6, add=FALSE,
                              ret_periods=c(2,5,10,15,20,25,30,40,50,75,100,200),
                              plot_extremes=TRUE) {
    # 12/block_size MUST BE INTENGER !
    durations <- c(1,2,3,6,12, c(1,2,3,4,5,7,10,15,20,25,30)*24)
    #durations <- c(6)
    #ret_periods <- c(100,200)
    palette <- turbo(length(ret_periods))
    if(add == FALSE) {
        plot(1, type = "n", xlab = "Durations", xaxt="n", yaxt="n",
             ylab = "Total mm",
             xlim = c(1, length(durations)),
             ylim = c(1, if (plot_intensities) {50} else {400} ),
             #main=paste(main,"-",if(force_weibull) {"Weibull fit"} else {"Free fit"},"- Block=",block_size,"months")
             main=paste(main,"- Block=",block_size,"months")
             )
        legend("topleft",
               legend=paste(ret_periods,"years"),
               col=palette,
               lty = 1)
        axis(side = 1, at = seq(length(durations)),labels = pretty_hours(hours(durations)))
        if (plot_intensities) {
            s <- seq(0,50,by=5)
            axis(side = 2, at = s,labels = s)
        } else  {
            s <- seq(0,400,by=50) # don't divide by ten
            axis(side = 2, at = s,labels = s)
        }
        # Plot the grid
        abline(h=seq(0,
                     if (plot_intensities) {50} else {400},
                     if (plot_intensities) {10} else {50}),col="grey")
        abline(v=seq(length(durations)), col="grey")
    }
    max_ret_lev <- c()
    min_ret_lev <- c()
    #par <-list(location=9.25964742, scale=4.39195400, shape=0.08594161 )
    ret_levels <- c()
    for(dndx in seq(length(durations))) {
        duration <- durations[dndx]
        ## d2 <- data.frame(
        ##     Date=full_ts$Date,
        ##     mm=c(filter(full_ts$mm, rep(1,duration), method="convolution", circular=TRUE)),
        ##     # Blocks are half years (mon-1:0..11 =>
        ##     block=year(full_ts$Date) + 0.5*floor((month(full_ts$Date)-1)/6))
        ## maxima <- blockmaxxer(d2, blocks=d2$block, which="mm")$mm
        maxima <- convolve_and_bm(full_ts, duration, block_size)
        print(paste(dndx,"/",length(durations)))
        ## print(maxima)
        ## print(tail(sort(full_ts$mm),10))
        ## fit <- fevd( maxima, type = "GEV", method = "Bayesian") #Lmoments
        ## par <- list(shape=mean(fit$results[,"shape"]),
        ##             location=mean(fit$results[,"location"]),
        ##             scale=exp(mean(fit$results[,"log.scale"])))
        fit <-  fevd( maxima, type = "GEV", method = "Lmoments") #Lmoments
        #par <- fit$results$par
        par <- list(location=fit$results["location"],
                    shape=fit$results["shape"],
                    scale=fit$results["scale"])
        if (force_weibull) {
            o <- optim(
                c(par[["location"]], scale=par[["scale"]],
                  if(par[["shape"]]<0) {par[["shape"]]} else {-0.2}),
                function(par) {
                    #print(par)
                    r <- sum(log(devd(maxima,loc=par[1],scale=par[2],shape=par[3])))
                    #print(r)
                    if( r == -Inf || is.nan(r)) {
                        r <- -1e200
                    }
                    return(r)
                },
                method="L-BFGS-B",
                lower=c(   0,    0.1,    -2),
                upper=c(160, 60,    -0.00000000001),
                control = list(fnscale=-1))# fnscale for maximisation)
            par <- list(location=o$par[1],scale=o$par[2],shape=o$par[3])
        }
        ## print("--------------------------")
        ## print(c(fit$results$par))
        ## print(unlist(par))
        ret_lev <- rlevd(
            ret_periods*(12/block_size), # Convert back to years
            loc=par[["location"]], scale=par[["scale"]], shape=par[["shape"]])
        if (plot_intensities) {
            ret_lev <- ret_lev / duration
        }
        ret_levels <- rbind(ret_levels, ret_lev)
        max_ret_lev <- c(max_ret_lev, max(ret_lev))
        min_ret_lev <- c(min_ret_lev, min(ret_lev))
        ## points(
        ##     rep(dndx, length(ret_lev)),
        ##     ret_lev,
        ##     col=palette)
        if (par[["shape"]] > 0) {
            text(dndx,-1,"+")
        } else  {
            text(dndx,-1,"-")
        }
        if (plot_extremes && plot_intensities == FALSE && max(ret_lev) < 500) {
            points(
                rep(dndx, length(maxima)),
                maxima,
                col=rgb(0,0,0,0.2),
                pch=20)
            density <- devd(seq(min(maxima),max(ret_lev)),
                            loc=par[["location"]], scale=par[["scale"]], shape=par[["shape"]])
            ## lines(dndx+0.5*density/max(density),
            ##       seq(min(maxima),max(ret_lev)))
        }
        qtiles <- c(0.05,0.25,0.50,0.75,0.95)
        tips <- c(4,0,15,0,4)
        pp <- c(filter(full_ts$mm, rep(1,duration), method="convolution", circular=TRUE))
        ## print("Quantiles")
        ## print(quantile(pp, probs=qtiles))
        points(rep(dndx, length(qtiles)),
               quantile(pp, probs=qtiles),
               col=rgb(1,0,0,0.5),
               pch=tips)
    }
    # Return levels lines
    for(i in seq(length(ret_periods))) {
        lines(t(ret_levels)[i,], col=palette[i])
    }
}



if(TRUE) {
for(shape in c("flat","triangle")) {
for(sid in c(BASTOGNE)) { #, ROBERTVILLE, WAVRE)) {
    sim_file_name <- toupper(stations_names[[paste(sid)]])
    observations <- extract_storms_hyetograms(c(sid))
    ## cnt <- round(sum(m)*0.1)
    ## full_ts_sim$mm[sample(which(m), cnt)] <- round(runif(cnt))*0.1
    # Plot IDF curves
    out_plot(paste("idf_sim_vs_obs_",sim_file_name,"_",shape,sep=""),width=16,height=8)
    par(mfrow=c(1,2))
    make_and_plot_idf(shrink_vec(observations$orig_tseries), main="Observation",  force_weibull=TRUE, plot_intensities=FALSE)
    fnames <- Sys.glob(paste("hyeto_",sim_file_name,"_",shape,"_*.csv",sep=""))
    for(fndx in seq(min(10,length(fnames)))) {
        full_ts_sim <- read_ts_for_idf(fnames[fndx])
        make_and_plot_idf(shrink_vec(full_ts_sim),
                          main=paste("Simulation",stations_names[[paste(sid)]]),
                          force_weibull=TRUE, plot_intensities=FALSE,
                          add=fndx != 1,
                          ret_periods=c(3,20,50,200),
                          plot_extremes=TRUE)
    }
    dev.off()
} # station
} # shape
} # if TRUE/FALSE


# .....................................................

# Running over several stations

N_SIM <- 10
PDF <- TRUE
shape <- "triangle"
library(reshape2)
all_errors <- list()
plots <- list()

done_sims <- c("BASTOGNE", "BATTICE", "BUTGENBACH", "CINEY", "EREZEE", "GEMMENICH", "JALHAY", "LANAYE", "LANDENNE", "LOUVEIGNE", "MARCHE", "MODAVE", "NASSOGNE", "ORTHO", "OUFFET", "RACHAMPS-NOVILLE", "ROBERTVILLE", "ROCHEFORT", "SANKT-VITH", "TAILLES", "TERNELL", "VIELSALM", "WAREMME", "WAVRE")



for(sim_file_name in  done_sims) { #, ROBERTVILLE, WAVRE)) { TEST_STATIONS_NAMES
    # Read original data
    observations <- extract_storms_hyetograms(string_to_station_id[[sim_file_name]])
    obs_idf <- compute_idf_projections(observations$orig_tseries, pdf=PDF)
    # Read simulations data
    all_sim_idf <- compute_all_idf(sim_file_name, shape, max_files=N_SIM, pdf=PDF)
    avg_idf <- Reduce("+",all_sim_idf) / length(all_sim_idf)
    error_idf <- abs(avg_idf - obs_idf) / obs_idf
    all_errors <- append(all_errors, list(error_idf))
    plots <- append(plots, list(ggplot(melt(error_idf), aes(Var1,Var2, fill=value)) + geom_raster()))
}


all_avg <- Reduce("+",all_errors) / length(all_errors)
sq <- lapply(all_errors, function (x) {(x - all_avg) ^ 2})
all_std <- Reduce("+",sq) / length(sq)
out_plot("all_idf_compare",width=16,height=8,res=150)
a <- ggplot(melt(all_avg), aes(Var1,Var2, fill=value)) + geom_raster() + labs(title="Relative Error, mean", x="Duration",y="Return Period")
b <- ggplot(melt(all_std), aes(Var1,Var2, fill=value)) + geom_raster() + labs(title="Relative Error, std",x="Duration",y="Return Period")
a + b
dev.off()


#######################################################

library(zoo)

sim_file_name <- "BASTOGNE"

out_plot(paste("hyeto_sim_vs_obs_",sim_file_name,sep=""),width=8,height=16,res=150)
observations <- extract_storms_hyetograms(BASTOGNE)
#shapes <- c("peaky","flat","triangle","optim_h","mcmc_h")
shapes <- c("flat","triangle","mcmc_h")
par(mfrow=c(length(shapes)+1,1))
s <- which.max(rollsum(observations$orig_tseries$mm,30*24))
e <- s + 30*24
mm_max <- max(observations$orig_tseries$mm[s:e])
plot(observations$orig_tseries$mm[s:e], type="l", main="Observations", ylim=c(0,mm_max), xlab="hour", ylab="mm")
for(shape in shapes) {
    fnames <- Sys.glob(paste("hyeto_",sim_file_name,"_",shape,"_00001.csv",sep=""))
    if(length(fnames) >= 1) {
        ## Compare simple hyetograms -------------------------------
        full_ts_sim <- read_ts_for_idf(fnames[1])
        s <- which.max(rollsum(full_ts_sim$mm,30*24))
        e <- s + 30*24
        plot(full_ts_sim$mm[s:e], type="l", main=paste("Simulations -",shape, "shape"),
             ylim=c(0,mm_max),
             xlab="hour", ylab="mm")
    }
}
dev.off()
#######################################################

sim_file_name <- "BASTOGNE"
observations <- extract_storms_hyetograms(BASTOGNE)
for(shape in c("peaky","flat","triangle","optim_h","mcmc_h")) {
    fnames <- Sys.glob(paste("hyeto_",sim_file_name,"_",shape,"_00001.csv",sep=""))
    if(length(fnames) >= 1) {
        # Showing hyetogram sub durations distributions for various hours -------------------------------
        # The noise around the bars is due to the noise that is reinjected
        # and which sometimes "pollutes" the extremities of a regular storm.
        out_plot(paste("subdurations_sim_",sim_file_name,"_",shape,sep=""),width=8,height=8)
        sim_storms_data <- extract_all_storms(full_ts_sim)
        sim_subdurations <- subdurations( sim_storms_data$all_long_storms)
        plot_subdurations_log_norm(sim_subdurations, start_plot=2, plot_pdf=TRUE)
        dev.off()
        # Comparing distributions of sub durations mm between obs and sim --------------------------
        obs_subdurations <- subdurations( observations$all_long_storms)
        sim_subdurations <- subdurations( sim_storms_data$all_long_storms)
        out_plot(paste("subdurations_dist_sim_vs_obs_",sim_file_name,"_",shape,sep=""),width=16,height=16,res=200)
        par(mfrow=c(3,3))
        for(i in seq(2,19,by=2)) {
            obs <- unlist(obs_subdurations[i])
            sim <- unlist(sim_subdurations[i])
            if(length(sim) < 10) {
                print("ERROR! Not enough sim data ???")
                next
            }
            qqplot(obs, sim,
                   main=paste(i,"h ,",length(obs),"obs /",length(sim)," sim.", sep=""),
                   xlab="Observation qtiles", ylab="Simulation qtiles")
            x <- quantile(obs,c(0.9,0.95,0.99))
            y <- quantile(sim,c(0.9,0.95,0.99))
            points(x,y,col="red",pch=8)
        }
        dev.off()
    } else {
        print(paste("Not enough simulations with shape", shape))
    }
}

## par(mfrow=c(2,1))
## e <- min(length(full_ts_sim$mm), length(observations$orig_tseries$mm))
## s <- e - 2000
## mm_max <- max(observations$orig_tseries$mm[s:e], full_ts_sim$mm[s:e])
## plot(observations$orig_tseries$mm[s:e], type="l", main="Observations", ylim=c(0,mm_max))
## plot(full_ts_sim$mm[s:e], type="l", main="Simulations", ylim=c(0,mm_max))

########################################################
## Check blck maxima

## maxima <- convolve_and_bm(shrink_vec(full_ts_sim), 1)
## maxima


qqplot_mm <- function(station_name, sim_storms, season="Summer", field="mm",...) {
    if(season == "Summer") {
        season_months <- SUMMER_MONTHS
    } else  {
        season_months <- WINTER_MONTHS
    }

    #  Read orignal data
    orig_data <- extract_storms_hyetograms(string_to_station_id[[station_name]])
    orig_storms <- orig_data$orig_storms[month(orig_data$orig_storms$Date) %in% season_months,]

    ylim_max <- 1
    for(i in seq(length(sim_storms))) {
        season_sim_data <- sim_storms[[i]][month(sim_storms[[i]]$Date) %in% season_months,field]
        ylim_max <- max(ylim_max, season_sim_data)
    }

    for(i in seq(length(sim_storms))) {
        if (i <= length(sim_storms)) {
            season_sim_data <- sim_storms[[i]][month(sim_storms[[i]]$Date) %in% season_months,field]
            my_qqplot(orig_storms[,field],
                      season_sim_data,
                      make.plot=i == 1,
                      col=rgb(0,0,0,0.05), pch=20,
                      xlim=c(0, max(orig_storms[,field])),
                      ylim=c(0, ylim_max),
                      main=paste(station_name,"-",season,"-",length(sim_storms),"sim."),...)
        }
    }
}


# QQPlot to compare distribution of storm's precipitation
out_plot("simulation_qqplot_mm",width=14,height=8,res=150)
par(mfrow=c(2,3))
for(station_name in TEST_STATIONS_NAMES) {
    qqplot_mm(station_name, all_storms[[station_name]], season="Summer", field="mm", xlab="mm observed", ylab="mm simulation")
}
for(station_name in TEST_STATIONS_NAMES) {
    qqplot_mm(station_name, all_storms[[station_name]], season="Winter", field="mm", xlab="mm observed", ylab="mm simulation")
}
dev.off()

out_plot("simulation_qqplot_durations",width=14,height=8,res=150)
par(mfrow=c(2,3))
for(station_name in TEST_STATIONS_NAMES) {
    qqplot_mm(station_name, all_storms[[station_name]], season="Summer", field="Duration", xlab="h observed", ylab="h simulation")
}
for(station_name in TEST_STATIONS_NAMES) {
    qqplot_mm(station_name, all_storms[[station_name]], season="Winter", field="Duration", xlab="h observed", ylab="h simulation")
}
dev.off()


total_water_per_season <- function(station_name) {
    # Simulations
    winter <- c()
    summer <- c()
    # very slow => use this to accelerate computations
    nsim <- min(20,length(all_time_series[[station_name]]))
    for(i in seq(nsim)) {
        storms <- all_time_series[[station_name]][[i]]
        print(paste(station_name, "i=",i, "rows:", nrow(all_time_series[[station_name]][[i]]),
                    "sum mm:", sum(storms$mm)))
        storms_years <- year(storms$Date)
        for(y in seq_range(year(storms$Date))) {
            m <- month(storms$Date)
            smm <- sum(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]$mm)
            wmm <- sum(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]$mm)
            if (smm > 100 && wmm > 100) {
                # Fixing not complete year
                summer <- c(summer, smm)
                winter <- c(winter, wmm)
                #print(paste("Simulation Year",y,"mm summer:",smm,"mm winter:",wmm))
            }
        }
    }
    # Observations
    winter_obs <- c()
    summer_obs <- c()
    orig_data <- extract_storms_hyetograms(string_to_station_id[[station_name]])
    storms <- orig_data$orig_tseries
    storms_years <- year(storms$Date)
    for(y in seq_range(year(orig_data$orig_storms$Date))) {
        m <- month(storms$Date)
        smm <- sum(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]$mm)
        summer_obs <- c(summer_obs, smm)
        wmm <- sum(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]$mm)
        winter_obs <- c(winter_obs, wmm)
        #print(paste("Orig Data: Year",y,"mm summer:",smm,"mm winter:",wmm))
    }
    r <- range(year(orig_data$orig_storms$Date))
    nb_years <- r[2]-r[1]+1
    df <- data.frame(
        summer_obs=c(summer_obs, rep(NA,length(summer) - length(summer_obs))),
        winter_obs=c(winter_obs, rep(NA,length(winter) - length(winter_obs))),
        summer=summer,
        winter=winter)
    boxplot(df,
            main=paste(station_name,nb_years,"years,",nsim,"simulations"),
            ylab="Total mm per year",
            xaxt="n")
    axis(side=1,at=c(1,2,3,4),labels=c("Summer - Obs","Winter - Obs","Summer - Sim","Winter - Sim"))
}


out_plot("simulation_water",width=8,height=8)

par(mfrow=c(2,2))
for(station_name in TEST_STATIONS_NAMES) {
    total_water_per_season(station_name)
}

dev.off()




storms_hours_per_season <- function(station_name, all_simulations) {
    # Simulated values
    winter <- c()
    summer <- c()
    for(i in seq(length(all_time_series[[station_name]]))) {
        storms <- simple_storm_separation(all_time_series[[station_name]][[i]])
        storms_years <- year(storms$Date)
        for(y in seq_range(year(storms$Date))) {
            m <- month(storms$Date)
            summer_s <- sum(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]$Duration)
            winter_s <- sum(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]$Duration)
            if (summer_s > 50 && winter_s > 50) {
                summer <- c(summer, summer_s)
                winter <- c(winter, winter_s)
            } else {
                print(paste("Dropping year",y,"; summer hours",summer_s,"; winter hours",winter_s))
            }
        }
    }
    # Observed values
    summer_obs <- c()
    winter_obs <- c()
    orig_data <- extract_storms_hyetograms(string_to_station_id[[station_name]])
    storms <- simple_storm_separation(orig_data$orig_tseries)
    #storms <- orig_data$orig_storms
    storms_years <- year(storms$Date)
    for(y in seq_range(year(orig_data$orig_storms$Date))) {
        m <- month(storms$Date)
        summer_s <- sum(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]$Duration)
        winter_s <- sum(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]$Duration)
        summer_obs <- c(summer_obs, summer_s)
        winter_obs <- c(winter_obs, winter_s)
    }
    r <- range(year(orig_data$orig_storms$Date))
    nb_years <- r[2]-r[1]+1
    boxplot(data.frame(
        # As I have obvioulsy less data for the observations than for the
        # simulation, I stuff with N/A.
        summer_obs=c(summer_obs, rep(NA,length(summer) - length(summer_obs))),
        winter_obs=c(winter_obs, rep(NA,length(winter) - length(winter_obs))),
        summer=summer,
        winter=winter),
        main=paste(station_name,nb_years,"years,",length(all_time_series[[station_name]]),"simulations"),
        ylab="Total rain hours per year",
        #ylim=c(0,700),
        xaxt="n")
    axis(side=1,at=c(1,2,3,4),labels=c("Summer - Obs","Winter - Obs","Summer - Sim","Winter - Sim"))
}


# Number of hours counted in storms
out_plot("simulation_storms_hours_per_season",width=8,height=8)

par(mfrow=c(2,2))
for(station_name in TEST_STATIONS_NAMES) {
    storms_hours_per_season(station_name,all_storms[[station_name]])
}

dev.off()

nb_storms_per_season <- function(station_name, all_simulations) {
    winter <- c()
    summer <- c()
    for(storms in all_simulations) {
        storms_years <- year(storms$Date)
        for(y in seq_range(year(storms$Date))) {
            m <- month(storms$Date)
            summer <- c(summer, nrow(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]))
            winter <- c(winter, nrow(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]))
        }
    }
    winter_obs <- c()
    summer_obs <- c()
    orig_data <- extract_storms_hyetograms(string_to_station_id[[station_name]])
    storms <- orig_data$orig_storms
    storms_years <- year(storms$Date)
    for(y in seq_range(year(orig_data$orig_storms$Date))) {
        m <- month(storms$Date)
        summer_obs <- c(summer_obs, nrow(storms[(m %in% SUMMER_MONTHS) & (storms_years == y),]))
        winter_obs <- c(winter_obs, nrow(storms[(m %in% WINTER_MONTHS) & (storms_years == y),]))
    }
    r <- range(year(orig_data$orig_storms$Date))
    nb_years <- r[2]-r[1]+1
    boxplot(data.frame(summer_obs=c(summer_obs, rep(NA,length(summer) - length(summer_obs))),
                       winter_obs=c(winter_obs, rep(NA,length(winter) - length(winter_obs))),
                       summer=summer,
                       winter=winter),
            main=paste(station_name,nb_years,"years,",length(all_simulations),"simulations"),
            ylab="Number of storms per year",
            xaxt="n")
    axis(side=1,at=c(1,2,3,4),labels=c("Summer - Obs","Winter - Obs","Summer - Sim","Winter - Sim"))
}

out_plot("simulation_storms_per_season",width=8,height=8)
par(mfrow=c(2,2))
for(station_name in TEST_STATIONS_NAMES) {
    nb_storms_per_season(station_name,all_storms[[station_name]])
}
dev.off()

#######################################################################################

copula_comparison_chart <- function(data_org, simulations_data, outname, sim_file_name, legend=TRUE) {

    helper <- function(data, title, legend=TRUE) {
        noise <- runif(nrow(data))
        df <- data.frame(x = round(data$Duration) + noise,
                         y = round(data$mm,1),
                         intensity = data$mm / data$Duration)
        top <- df[tail(order(df$intensity),10),]
        plot <- ggplot(df, aes(x,y)) +
            stat_binhex(bins=69) + # Carefully selected to hide the discretization effect
            #scale_fill_gradient(name = "count") +
            xlim(XLIM) + ylim(YLIM) +
            scale_fill_viridis_c(limits = CLIM) +
            geom_hline(yintercept=35, linetype='dashed') +
            geom_vline(xintercept=24, linetype='dashed',) +
            geom_point(data=top, aes(x,y), color="red", shape=17, size=3)

        if (legend == FALSE) {
            plot <- plot + labs(title=title)
            #plot <- plot + scale_x_discrete(labels = NULL, breaks = NULL) + labs(x = "")
            plot <- plot + theme(legend.position = "none",
                                 axis.text.x=element_blank(), #remove x axis labels
                                 axis.ticks.x=element_blank(), #remove x axis ticks
                                 axis.title.x=element_blank(),
                                 axis.text.y=element_blank(),  #remove y axis labels
                                 axis.ticks.y=element_blank(),  #remove y axis ticks
                                 axis.title.y=element_blank()
                                 )
        } else {
            plot <- plot + labs(x="Duration (h)",y="Precipitation (mm)", title=title)
        }
        return(plot)
    }

    par(mfrow=c(1,4))
    print(paste("copula_comparison_chart()",nrow(data_org),"storms"))
    print(paste("# simulations",length(simulations_data)))
    max_dur <- max(data_org$Duration)
    max_mm <- max(data_org$mm)
    for(i in seq(length(simulations_data))) {
        max_dur <- max(max_dur, max(simulations_data[[i]]$Duration))
        max_mm <- max(max_mm, max(simulations_data[[i]]$mm))
    }

    XLIM <- c(0,max_dur)
    YLIM <- c(0,max(data_org$mm)*1.2)
    CLIM <- c(0,50)
    # The first chart is the real data
    plot <- helper(data_org,paste(sim_file_name,nrow(data_org),"storms"))
    charts <- list()
    charts[[1]] <- plot
    for (i in seq(length(simulations_data))) {
        samples <- simulations_data[[i]]
        plot <- helper(samples,paste("Simulation",i,"-",nrow(samples),"storms"),legend=FALSE)
        charts[[i+1]] <- plot
    }
    print( (charts[[1]] + charts[[2]]) / (charts[[3]] + charts[[4]]) / (charts[[5]] + charts[[6]]))
}

#out_plot(outname,width=8,height=8)

print("copula_comparison")
for(station_name in c("BASTOGNE", "ROBERTVILLE","WAVRE")) {
    out_plot(paste("copula_comparison_",station_name,sep=""),width=8,height=8*1.5)
    fnames <- Sys.glob(paste("hyeto_",station_name,"_","flat","_*.csv",sep=""))
    if (length(fnames) >= 3) {
        storms1 <- simple_storm_separation(read_ts_for_idf(fnames[1]))
        storms2 <- simple_storm_separation(read_ts_for_idf(fnames[2]))
        storms3 <- simple_storm_separation(read_ts_for_idf(fnames[3]))
        storms4 <- simple_storm_separation(read_ts_for_idf(fnames[4]))
        storms5 <- simple_storm_separation(read_ts_for_idf(fnames[5]))
        orig_data <- extract_storms_hyetograms(string_to_station_id[[station_name]])
        copula_comparison_chart(orig_data$orig_storms, list(storms1, storms2, storms3, storms4, storms5),
                                paste("copula_comparison_",station_name,sep=""),
                                station_name)
    } else {
        print(paste("Missing data for", station_name, "I need at least 3 simulations per station"))
    }
    dev.off()
}

########################################################################
## The QQPlot (duration, mm) gives information on the quality of the fit.
## But not about the varaibility of the simulation (the graphs are cluttered).

## # Storm mm fro a given duration analysis
## # (think mean IDF rather than extreme IDF)
## # We should do that on hyetograms, just like regular IDF curves.
## # But if we do it means that computation will take ages if we go for a hundred simulations.
## # This will allow to validate the copula's marginals

if (!interactive()) {x11(width=12, height=12)}

out_plot("mm_per_duration_quantiles",width=8,height=8)

par(mfrow=c(2,2))
for(station_name in TEST_STATIONS_NAMES) {
    if(!(length(all_storms[[station_name]]) > 0)) {
        print(paste("No data for", station_name))
        next
    }
    # Join all storms of all the simulation together
    storms <- data.frame()
    for (s in all_storms[[station_name]]) {
        storms <- rbind(storms, s)
    }
    quantiles <- c(0,0.01,0.05,0.1,0.45,0.55,0.9,0.95,0.99,1)
    qtiles <- data.frame()
    for(duration in seq(max(storms$Duration))) {
        mms <- storms[storms$Duration == duration,c("mm")]
        qtiles <- rbind(qtiles,
                        quantile(mms, probs=quantiles) ) # / max(mms)
    }
    colnames(qtiles) <- quantiles
    # I've tried with ggplot, it's a mess.
    print(paste(station_name,
                length(all_storms[[station_name]]),"sim.,",
                length(unique(year(storms$Date))),"years each"))
    print(qtiles)
    plot(qtiles[,1],
         xlab="Duration (h)", ylab="Precipitation (mm)", type="l",
         main=paste(station_name,
                    length(all_storms[[station_name]]),"sim.,",
                    length(unique(year(storms$Date))),"years each"))
    pal <- c(rgb(1,0.9,0.9), rgb(1,0.8,0.8), rgb(1,0.4,0.4), rgb(1,0,0), rgb(0.5,0,0), rgb(1,0,0),
             rgb(1,0.4,0.4), rgb(1,0.8,0.8), rgb(1,0.9,0.9))
    for( i in seq(2,length(quantiles))) {
        lines(qtiles[,i], xlab="Duration (h)", ylab="Precipitation (mm)", type="l")
        polygon(x=c(rev(seq(nrow(qtiles))), seq(nrow(qtiles))),
                y=c(rev(qtiles[,i]), qtiles[,i-1]),
                col=pal[i-1])
    }
    # Plot legend
    l <- c()
    for(i in rev(seq(length(quantiles)-1))) {
        l <- c(l, paste(quantiles[i]*100, "-",quantiles[i+1]*100,"%",sep=""))
    }
    legend(x="topleft", legend=l, fill=pal)
}

dev.off()




# .............................................................

out_plot("bare_mm_distribution",width=8,height=8)
MAX_MM <- 10
stat_mms <- function(mms) {
    #mms <- mms[mms >= NOISE_LEVEL]
    h <- hist(mms[mms <= MAX_MM], breaks=seq(0,MAX_MM*10)*0.1, plot=FALSE)
    list(x=h$breaks[1:(length(h$breaks)-1)],
         y=h$counts*h$breaks[1:(length(h$breaks)-1)])
}
par(mfrow=c(2,2))
for(name in TEST_STATIONS_NAMES) {
    #name <- "ROBERTVILLE" # "WAVRE" #"BASTOGNE"
    orig_data <- extract_storms_hyetograms(string_to_station_id[[name]])
    mms <- orig_data$orig_tseries$mm
    print(sum(mms))
    s <- stat_mms(mms)
    plot(1, type = "n",
         xlab="Hourly rainfall (mm)",
         ylab="Total rainfall (mm)",
         xlim=c(0,MAX_MM),
         ylim=c(0,max(stat_mms(all_time_series[[name]][[1]]$mm)$y)),
         main=paste(name,"-",length(all_time_series[[name]]), "simulations"))
    tot_sim <- rep(0,MAX_MM*10)
    for( i in seq(length(all_time_series[[name]]))) {
        mms <- all_time_series[[name]][[i]]$mm
        s <- stat_mms(mms)
        tot_sim <- tot_sim + s$y
        lines(s$x, s$y, col=rgb(0,0,0,.1))
    }
    mms <- orig_data$orig_tseries$mm
    s <- stat_mms(mms)
    lines(s$x,s$y, col="red", lw=2)
    lines(s$x,tot_sim/length(all_time_series[[name]]), col="green",lw=2)
}
dev.off()

# It can be rounding (when we round small flat storms, we
# probably create tons of errors)

# .............................................................

plot_subduration_obs_vs_sim <- function(start_plot, cuts_org, precomputed_cuts, name) {
    ## duration_counts <- rep(0,100)
    ## for( sim in seq(length(precomputed_cuts))) {
    ##     for( d in seq(length(precomputed_cuts[[sim]]))) {
    ##         duration_counts[d] <- length(precomputed_cuts[[sim]][[d]])
    ##     }
    ## }

    par(mfrow=c(3,3))
    for( duration in seq(start_plot,start_plot+9*2-1,by=2)) {
        for( i in seq(length(precomputed_cuts))) {
            if(length(precomputed_cuts[[i]]) < duration) {
                # In Wavre there are not very long storms
                return()
            }
            my_qqplot(cuts_org[[duration]],
                      precomputed_cuts[[i]][[duration]],
                      make.plot = i==1,
                      main=paste(duration,"h. -",length(precomputed_cuts),"sim.",name),
                      #",",round(100*duration_counts[duration]/sum(duration_counts),1),"%"
                      xlab="Observation quantiles (mm)",
                      ylab="Simulation quantiles (mm)",
                      col=rgb(0,0,0,0.1),
                      pch=20)
        }
    }
}

# Here we'd need mcmc to display something meaningful
shape <- "flat"

all_time_series_flat <- read_all_simulations_time_series("flat")
all_time_series_triangle <- read_all_simulations_time_series("triangle")
all_time_series_mcmc_h <- read_all_simulations_time_series("mcmc_h")
orig_data <- extract_storms_hyetograms(string_to_station_id[[name]])
z <- extract_all_storms(orig_data$orig_tseries)
cuts_org <- subdurations(z$all_long_storms)

for(name in c("WAVRE","ROBERTVILLE")) {
N_SIM=10
all_sim_cuts <- list()
for(sim in seq(N_SIM)) {
    storms <- extract_all_storms(all_time_series_flat[[name]][[sim]])
    sim_cuts <- subdurations(storms$all_long_storms)
    all_sim_cuts <- append(all_sim_cuts, list(sim_cuts))
}
all_sim_cuts_flat <- all_sim_cuts
all_sim_cuts <- list()
for(sim in seq(N_SIM)) {
    storms <- extract_all_storms(all_time_series_triangle[[name]][[sim]])
    sim_cuts <- subdurations(storms$all_long_storms)
    all_sim_cuts <- append(all_sim_cuts, list(sim_cuts))
}
all_sim_cuts_triangle <- all_sim_cuts
all_sim_cuts <- list()
for(sim in seq(N_SIM)) {
    storms <- extract_all_storms(all_time_series_mcmc_h[[name]][[sim]])
    sim_cuts <- subdurations(storms$all_long_storms)
    all_sim_cuts <- append(all_sim_cuts, list(sim_cuts))
}
all_sim_cuts_mcmc_h <- all_sim_cuts
out_plot(paste("shape_compare_",name,sep=""),  width=16,height=16, res=300)
par(mfrow=c(3,3),mar = c(0.1, 0.1, 0.1, 0.1))
for( d in seq(9)*2) {
    t_flat <- lapply(all_sim_cuts_flat, function (cut) {sort(cut[[d]])})
    t_triangle <- lapply(all_sim_cuts_triangle, function (cut) {sort(cut[[d]])})
    t_mcmc_h <- lapply(all_sim_cuts_mcmc_h, function (cut) {sort(cut[[d]])})
    my_qqplot(cuts_org[[d]],
              unlist(t_flat),
              make.plot = TRUE,
              #",",round(100*duration_counts[duration]/sum(duration_counts),1),"%"
              ## xlab="Observation quantiles (mm)",
              ## ylab="Simulation quantiles (mm)",
              xaxt="n", yaxt="n", xlab="", ylab="",
              col=rgb(1,0,0,0.5),
              pch=20)
    my_qqplot(cuts_org[[d]], unlist(t_triangle), make.plot = FALSE, col=rgb(0,1,0,0.5), pch=20)
    my_qqplot(cuts_org[[d]], unlist(t_mcmc_h),   make.plot = FALSE, col=rgb(0,0,1,0.5), pch=20)
    text(x=max(cuts_org[[d]])/2,
         y=min(cuts_org[[d]]),paste(d,"hours"))
    if(d == 2) {
        legend(x="topleft",
               legend=c("Flat","Triangle","MCMC"),
               pch=20,
               col=c(rgb(1,0,0),rgb(0,1,0),rgb(0,0,1)))
    }
}
dev.off()
}

for( name in c("ROBERTVILLE")) {

    name <- "BASTOGNE" # "ROBERTVILLE" # ,
    orig_data <- extract_storms_hyetograms(string_to_station_id[[name]])
    z <- extract_all_storms(orig_data$orig_tseries)
    cuts_org <- subdurations(z$all_long_storms)
    # Precomputing the subdurations cuts for all simulation.
    all_sim_cut <- list()
    for( i in seq(min(10,length(all_time_series[[name]])))) {
        print(paste("Precomputing subduration cuts for simulation",i,name))
        z <- extract_all_storms(all_time_series[[name]][[i]])
        cuts <- subdurations(z$all_long_storms)
        all_sim_cut <- append(all_sim_cut, list(cuts))
    }

    out_plot(paste("subduration_obs_vs_sim_",shape,"_",name,sep=""),width=16,height=16,res=200)
    plot_subduration_obs_vs_sim(2, cuts_org, all_sim_cut, name)
    dev.off()

    out_plot(paste("subduration_obs_vs_sim2_",shape,"_",name,sep=""),width=8,height=8,res=150)
    plot_subduration_obs_vs_sim(2+9*2, cuts_org, all_sim_cut, name)
    dev.off()

}

# .......................................................................

out_plot("bare_duration_distribution",width=8,height=8)

MAX_MM <- 30
stat_mms <- function(mms) {
    #mms <- mms[mms >= NOISE_LEVEL]
    # With right = FALSE this is equivalent to table!
    h <- hist(mms[mms <= MAX_MM], breaks=seq(1,MAX_MM+1), plot=FALSE, right = FALSE)
    list(x=h$breaks[1:(length(h$breaks)-1)],
         y=h$counts #* h$breaks[1:(length(h$breaks)-1)]
         )
}

par(mfrow=c(2,2))
for(name in TEST_STATIONS_NAMES) {
    #name <- "ROBERTVILLE" # "WAVRE" #"BASTOGNE"
    orig_data <- extract_storms_hyetograms(string_to_station_id[[name]])
    mms <- orig_data$orig_storms$Duration
    s <- stat_mms(mms)
    plot(1, type = "n",
         xlab="Storm duration (h)",
         ylab="# of storms",
         xlim=c(0,MAX_MM),
         ylim=c(0,max(stat_mms(all_storms[[name]][[1]]$Duration)$y)),
         main=paste(name,"-",length(all_storms[[name]]), "simulations"))
    tot_sim <- rep(0,MAX_MM)
    for( i in seq(length(all_storms[[name]]))) {
        mms <- all_storms[[name]][[i]]$Duration
        s <- stat_mms(mms)
        tot_sim <- tot_sim + s$y
        lines(s$x, s$y, col=rgb(0,0,0,.1))
    }
    mms <- orig_data$orig_storms$Duration
    s <- stat_mms(mms)
    lines(s$x,s$y, col="red", lw=2)
    lines(s$x,tot_sim/length(all_storms[[name]]), col="green",lw=2)
}

dev.off()


# ---------------------------------------------------------------------------------------



storms <- read_storm_file("simstorm_BASTOGNE_00010.csv")
head(storms)

bin2d <- function(xy, xbreaks, ybreaks) {
    x.bin <- xbreaks
    y.bin <- ybreaks
    freq <-  as.data.frame(table(findInterval(xy[,1], x.bin),
                                 findInterval(xy[,2], y.bin)))
    freq[,1] <- as.numeric(freq[,1])
    freq[,2] <- as.numeric(freq[,2])
    freq2D <- diag(max(length(x.bin),length(y.bin)))*0
    freq2D[cbind(freq[,1], freq[,2])] <- freq[,3]
    return(freq2D)
}

done_sims <- c("BASTOGNE", "BATTICE", "BUTGENBACH", "CINEY", "EREZEE", "GEMMENICH", "JALHAY", "LANAYE", "LANDENNE", "LOUVEIGNE", "MARCHE", "MODAVE", "NASSOGNE", "ORTHO", "OUFFET", "RACHAMPS-NOVILLE", "ROBERTVILLE", "ROCHEFORT", "SANKT-VITH", "TAILLES", "TERNELL", "VIELSALM", "WAREMME", "WAVRE")

done_sims <- c("BASTOGNE")

all_errors <- list()
all_orgs <- list()
for(station_name in done_sims) {
    storms_org <- read_storm_file(paste("storms_",station_name,".csv",sep=""))
    fnames <- Sys.glob(paste("simstorm_",station_name,"*.csv",sep=""))
    all_storms <- list()
    for(fname in fnames) {
        storms <- read_storm_file(fname)
        all_storms <- append( all_storms, list(bin2d(storms[,c("Duration","mm")], seq(0,100), seq(0,120))))
    }
    m_org <- bin2d(storms_org[,c("Duration","mm")], seq(0,100), seq(0,120))
    average_prediction <- Reduce("+",all_storms) / length(all_storms)
    m <- abs(average_prediction - m_org) # / m_org
    all_errors <- append( all_errors, list(m))
    all_orgs <- append( all_orgs, list(m_org))
}


m_org <- Reduce("+",all_orgs)
mask <- m_org<10
percent_errors <- lapply(all_errors, function(x) {
    x[mask] <- NA
    100*x/m_org
    })



percent_errors <- list()
for(i in seq(length(all_errors))) {
    percent_errors <- append(percent_errors, list(all_errors[[i]]/all_orgs[[i]]))
}

mean_stations <- Reduce("+",percent_errors) / length(percent_errors)
df <- melt(mean_stations)
df$value <- as.factor(round(round(df$value)/10)*10)
#out_plot("all_nb_storms_errors", width=8,height=8, res=150)
mean_plot <- ggplot(df, aes(x=Var1, y=Var2, fill=value)) +
    geom_tile() +
    scale_fill_viridis_d(limits=rev) + #breaks = my_breaks, labels = paste(round(my_breaks*100),"%",sep="")) +
    xlim(0,20) + ylim(0,25) + xlab("Duration (mm)") + ylab("Rainfall (mm)") + labs(title="Mean")


sq <- lapply(all_errors, function (x) {(x - mean_stations) ^ 2})
std_stations <- sqrt(Reduce("+",sq) / length(sq))
df <- melt(std_stations)
std_plot <- ggplot(df, aes(x=Var1, y=Var2, fill=value)) +
    geom_tile() +
    scale_fill_viridis_c() + #breaks = my_breaks, labels = paste(round(my_breaks*100),"%",sep="")) +
    xlim(0,20) + ylim(0,25) + xlab("Duration (mm)") + ylab("Rainfall (mm)") + labs(title="Standard Deviation")

mean_plot + std_plot

#dev.off()




s <- 0
for(i in seq(length(all_errors))) {
    s <- s + all_orgs[[i]]
}
s <- s/length(all_errors)
print(round(s[1:10,1:10]))

s <- 0
for(i in seq(length(all_errors))) {
    s <- s + round(100*all_errors[[i]] / (1+all_orgs[[i]]))
}
mean_errors <- (s/length(all_errors))
print(round(mean_errors[1:10,1:10]))

breaks <- c(0,10,20,30,40,50,100,200,300,400,500)
s <- matrix(findInterval(mean_errors, breaks), nrow=nrow(all_errors[[1]]))
ms <- melt(s)
print(unique(ms$value))
ms$value <- as.factor(ms$value)
out_plot("all_nb_storms_errors", width=10,height=8, res=150)
ggplot(ms, aes(x=Var1, y=Var2, fill=value)) +
    geom_tile() +
    scale_fill_discrete(labels=c("0-10%","10-20%","20-30%","30-40%","40-50%","50-100%","100-200%","200-300%","300-400%","400-500%","500-600%")) +
    xlim(0,30) + ylim(0,60) + xlab("Duration (h)") + ylab("Rainfall (mm)") +
    geom_rect(xmin = 0.5, xmax = 11.5,   ymin = 0.5, ymax = 11.5, fill=NA, color="black")
dev.off()
